# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(378, 531)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setDocumentMode(False)
        self.tabWidget.setTabsClosable(False)
        self.tabWidget.setTabBarAutoHide(False)
        self.tabWidget.setObjectName("tabWidget")
        self.gridLayout.addWidget(self.tabWidget, 1, 0, 1, 1)
        self.label_title = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 378, 22))
        self.menubar.setObjectName("menubar")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionCurrent_alignment = QtWidgets.QAction(MainWindow)
        self.actionCurrent_alignment.setObjectName("actionCurrent_alignment")
        self.actionManipulator_Positions_Calibration = QtWidgets.QAction(MainWindow)
        self.actionManipulator_Positions_Calibration.setObjectName("actionManipulator_Positions_Calibration")
        self.actionMinimap = QtWidgets.QAction(MainWindow)
        self.actionMinimap.setObjectName("actionMinimap")
        self.actionOpen_Minimap = QtWidgets.QAction(MainWindow)
        self.actionOpen_Minimap.setObjectName("actionOpen_Minimap")
        self.actionImage_Viewer = QtWidgets.QAction(MainWindow)
        self.actionImage_Viewer.setObjectName("actionImage_Viewer")
        self.menuTools.addAction(self.actionManipulator_Positions_Calibration)
        self.menuTools.addAction(self.actionOpen_Minimap)
        self.menubar.addAction(self.menuTools.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label_title.setText(_translate("MainWindow", "OpenFIBSEM"))
        self.menuTools.setTitle(_translate("MainWindow", "Tools"))
        self.actionCurrent_alignment.setText(_translate("MainWindow", "Beam Current Alignment"))
        self.actionManipulator_Positions_Calibration.setText(_translate("MainWindow", "Manipulator Callibration"))
        self.actionMinimap.setText(_translate("MainWindow", "Minimap"))
        self.actionOpen_Minimap.setText(_translate("MainWindow", "Open Minimap"))
        self.actionImage_Viewer.setText(_translate("MainWindow", "Image Viewer"))
