# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemMicroscopeConfigurationWidgetBase.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(530, 884)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(Form)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.scrollArea.setFont(font)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 496, 1397))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lineEdit_electron_detector_mode = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_electron_detector_mode.setFont(font)
        self.lineEdit_electron_detector_mode.setObjectName("lineEdit_electron_detector_mode")
        self.gridLayout_2.addWidget(self.lineEdit_electron_detector_mode, 27, 1, 1, 2)
        self.doubleSpinBox_ion_eucentric_height = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_eucentric_height.setFont(font)
        self.doubleSpinBox_ion_eucentric_height.setObjectName("doubleSpinBox_ion_eucentric_height")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_eucentric_height, 32, 1, 1, 2)
        self.label_electron_detector_mode = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_detector_mode.setFont(font)
        self.label_electron_detector_mode.setObjectName("label_electron_detector_mode")
        self.gridLayout_2.addWidget(self.label_electron_detector_mode, 27, 0, 1, 1)
        self.label_configuration_path = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_path.setFont(font)
        self.label_configuration_path.setObjectName("label_configuration_path")
        self.gridLayout_2.addWidget(self.label_configuration_path, 5, 0, 1, 1)
        self.label_ion_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_hfw.setFont(font)
        self.label_ion_hfw.setObjectName("label_ion_hfw")
        self.gridLayout_2.addWidget(self.label_ion_hfw, 36, 0, 1, 1)
        self.label_configuration_name = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_name.setFont(font)
        self.label_configuration_name.setObjectName("label_configuration_name")
        self.gridLayout_2.addWidget(self.label_configuration_name, 4, 0, 1, 1)
        self.label_milling_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_milling_header.setFont(font)
        self.label_milling_header.setObjectName("label_milling_header")
        self.gridLayout_2.addWidget(self.label_milling_header, 47, 0, 1, 3)
        self.doubleSpinBox_electron_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_current.setFont(font)
        self.doubleSpinBox_electron_current.setMaximum(100000000.0)
        self.doubleSpinBox_electron_current.setObjectName("doubleSpinBox_electron_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_current, 23, 1, 1, 2)
        self.spinBox_electron_column_tilt = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_electron_column_tilt.setFont(font)
        self.spinBox_electron_column_tilt.setMaximum(180)
        self.spinBox_electron_column_tilt.setObjectName("spinBox_electron_column_tilt")
        self.gridLayout_2.addWidget(self.spinBox_electron_column_tilt, 17, 1, 1, 2)
        self.checkBox_subsystems_ion_plasma = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_ion_plasma.setFont(font)
        self.checkBox_subsystems_ion_plasma.setObjectName("checkBox_subsystems_ion_plasma")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_ion_plasma, 30, 1, 1, 1)
        self.label_milling_spot_size = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_spot_size.setFont(font)
        self.label_milling_spot_size.setObjectName("label_milling_spot_size")
        self.gridLayout_2.addWidget(self.label_milling_spot_size, 53, 0, 1, 1)
        self.doubleSpinBox_milling_rate = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_rate.setFont(font)
        self.doubleSpinBox_milling_rate.setMaximum(10000000.0)
        self.doubleSpinBox_milling_rate.setObjectName("doubleSpinBox_milling_rate")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_rate, 52, 1, 1, 2)
        self.checkBox_subsystems_stage_tilt = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_stage_tilt.setFont(font)
        self.checkBox_subsystems_stage_tilt.setObjectName("checkBox_subsystems_stage_tilt")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_stage_tilt, 10, 2, 1, 1)
        self.checkBox_imaging_autogamma = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_imaging_autogamma.setFont(font)
        self.checkBox_imaging_autogamma.setObjectName("checkBox_imaging_autogamma")
        self.gridLayout_2.addWidget(self.checkBox_imaging_autogamma, 46, 1, 1, 1)
        self.label_imaging_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_dwell_time.setFont(font)
        self.label_imaging_dwell_time.setObjectName("label_imaging_dwell_time")
        self.gridLayout_2.addWidget(self.label_imaging_dwell_time, 44, 0, 1, 1)
        self.doubleSpinBox_imaging_hfw = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_imaging_hfw.setFont(font)
        self.doubleSpinBox_imaging_hfw.setDecimals(0)
        self.doubleSpinBox_imaging_hfw.setMaximum(100000.0)
        self.doubleSpinBox_imaging_hfw.setObjectName("doubleSpinBox_imaging_hfw")
        self.gridLayout_2.addWidget(self.doubleSpinBox_imaging_hfw, 43, 1, 1, 2)
        self.label_ion_detector_mode = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_detector_mode.setFont(font)
        self.label_ion_detector_mode.setObjectName("label_ion_detector_mode")
        self.gridLayout_2.addWidget(self.label_ion_detector_mode, 38, 0, 1, 1)
        self.lineEdit_configuration_ip_address = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_configuration_ip_address.setFont(font)
        self.lineEdit_configuration_ip_address.setObjectName("lineEdit_configuration_ip_address")
        self.gridLayout_2.addWidget(self.lineEdit_configuration_ip_address, 7, 1, 1, 2)
        self.lineEdit_ion_detector_type = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_ion_detector_type.setFont(font)
        self.lineEdit_ion_detector_type.setObjectName("lineEdit_ion_detector_type")
        self.gridLayout_2.addWidget(self.lineEdit_ion_detector_type, 39, 1, 1, 2)
        self.label_imaging_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_resolution.setFont(font)
        self.label_imaging_resolution.setObjectName("label_imaging_resolution")
        self.gridLayout_2.addWidget(self.label_imaging_resolution, 42, 0, 1, 1)
        self.checkBox_subsystems_stage_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_stage_enabled.setFont(font)
        self.checkBox_subsystems_stage_enabled.setObjectName("checkBox_subsystems_stage_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_stage_enabled, 10, 0, 1, 1)
        self.label_milling_rate = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_rate.setFont(font)
        self.label_milling_rate.setObjectName("label_milling_rate")
        self.gridLayout_2.addWidget(self.label_milling_rate, 52, 0, 1, 1)
        self.label_electron_column_tilt = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_column_tilt.setFont(font)
        self.label_electron_column_tilt.setObjectName("label_electron_column_tilt")
        self.gridLayout_2.addWidget(self.label_electron_column_tilt, 17, 0, 1, 1)
        self.spinBox_stage_shuttle_pre_tilt = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_stage_shuttle_pre_tilt.setFont(font)
        self.spinBox_stage_shuttle_pre_tilt.setMaximum(180)
        self.spinBox_stage_shuttle_pre_tilt.setObjectName("spinBox_stage_shuttle_pre_tilt")
        self.gridLayout_2.addWidget(self.spinBox_stage_shuttle_pre_tilt, 13, 1, 1, 2)
        self.label_electron_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_hfw.setFont(font)
        self.label_electron_hfw.setObjectName("label_electron_hfw")
        self.gridLayout_2.addWidget(self.label_electron_hfw, 25, 0, 1, 1)
        self.spinBox_stage_rotation_180 = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_stage_rotation_180.setFont(font)
        self.spinBox_stage_rotation_180.setMinimum(-360)
        self.spinBox_stage_rotation_180.setMaximum(360)
        self.spinBox_stage_rotation_180.setObjectName("spinBox_stage_rotation_180")
        self.gridLayout_2.addWidget(self.spinBox_stage_rotation_180, 12, 1, 1, 2)
        self.label_ion_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_voltage.setFont(font)
        self.label_ion_voltage.setObjectName("label_ion_voltage")
        self.gridLayout_2.addWidget(self.label_ion_voltage, 33, 0, 1, 1)
        self.doubleSpinBox_milling_voltage = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_voltage.setFont(font)
        self.doubleSpinBox_milling_voltage.setDecimals(0)
        self.doubleSpinBox_milling_voltage.setMaximum(10000000.0)
        self.doubleSpinBox_milling_voltage.setObjectName("doubleSpinBox_milling_voltage")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_voltage, 48, 1, 1, 2)
        self.label_milling_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_dwell_time.setFont(font)
        self.label_milling_dwell_time.setObjectName("label_milling_dwell_time")
        self.gridLayout_2.addWidget(self.label_milling_dwell_time, 51, 0, 1, 1)
        self.label_electron_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_dwell_time.setFont(font)
        self.label_electron_dwell_time.setObjectName("label_electron_dwell_time")
        self.gridLayout_2.addWidget(self.label_electron_dwell_time, 26, 0, 1, 1)
        self.label_electron_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_electron_header.setFont(font)
        self.label_electron_header.setObjectName("label_electron_header")
        self.gridLayout_2.addWidget(self.label_electron_header, 15, 0, 1, 3)
        self.doubleSpinBox_ion_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_current.setFont(font)
        self.doubleSpinBox_ion_current.setMaximum(10000000000.0)
        self.doubleSpinBox_ion_current.setObjectName("doubleSpinBox_ion_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_current, 34, 1, 1, 2)
        self.doubleSpinBox_milling_spot_size = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_spot_size.setFont(font)
        self.doubleSpinBox_milling_spot_size.setMaximum(10000000.0)
        self.doubleSpinBox_milling_spot_size.setObjectName("doubleSpinBox_milling_spot_size")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_spot_size, 53, 1, 1, 2)
        self.spinBox_stage_rotation_reference = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_stage_rotation_reference.setFont(font)
        self.spinBox_stage_rotation_reference.setMinimum(-360)
        self.spinBox_stage_rotation_reference.setMaximum(360)
        self.spinBox_stage_rotation_reference.setObjectName("spinBox_stage_rotation_reference")
        self.gridLayout_2.addWidget(self.spinBox_stage_rotation_reference, 11, 1, 1, 2)
        self.label_configuration_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_configuration_header.setFont(font)
        self.label_configuration_header.setObjectName("label_configuration_header")
        self.gridLayout_2.addWidget(self.label_configuration_header, 3, 0, 1, 1)
        self.doubleSpinBox_ion_voltage = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_voltage.setFont(font)
        self.doubleSpinBox_ion_voltage.setDecimals(0)
        self.doubleSpinBox_ion_voltage.setMaximum(100000000000.0)
        self.doubleSpinBox_ion_voltage.setObjectName("doubleSpinBox_ion_voltage")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_voltage, 33, 1, 1, 2)
        self.label_ion_detector_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_detector_type.setFont(font)
        self.label_ion_detector_type.setObjectName("label_ion_detector_type")
        self.gridLayout_2.addWidget(self.label_ion_detector_type, 39, 0, 1, 1)
        self.label_imaging_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_current.setFont(font)
        self.label_imaging_current.setObjectName("label_imaging_current")
        self.gridLayout_2.addWidget(self.label_imaging_current, 45, 0, 1, 1)
        self.checkBox_imaging_autosave = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_imaging_autosave.setFont(font)
        self.checkBox_imaging_autosave.setObjectName("checkBox_imaging_autosave")
        self.gridLayout_2.addWidget(self.checkBox_imaging_autosave, 46, 2, 1, 1)
        self.label_imaging_beam_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_beam_type.setFont(font)
        self.label_imaging_beam_type.setObjectName("label_imaging_beam_type")
        self.gridLayout_2.addWidget(self.label_imaging_beam_type, 41, 0, 1, 1)
        self.checkBox_imaging_autocontrast = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_imaging_autocontrast.setFont(font)
        self.checkBox_imaging_autocontrast.setObjectName("checkBox_imaging_autocontrast")
        self.gridLayout_2.addWidget(self.checkBox_imaging_autocontrast, 46, 0, 1, 1)
        self.checkBox_subsystems_electron_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_electron_enabled.setFont(font)
        self.checkBox_subsystems_electron_enabled.setObjectName("checkBox_subsystems_electron_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_electron_enabled, 16, 0, 1, 1)
        self.label_stage_rotation_180 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_rotation_180.setFont(font)
        self.label_stage_rotation_180.setObjectName("label_stage_rotation_180")
        self.gridLayout_2.addWidget(self.label_stage_rotation_180, 12, 0, 1, 1)
        self.label_imaging_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_imaging_hfw.setFont(font)
        self.label_imaging_hfw.setObjectName("label_imaging_hfw")
        self.gridLayout_2.addWidget(self.label_imaging_hfw, 43, 0, 1, 1)
        self.doubleSpinBox_electron_voltage = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_voltage.setFont(font)
        self.doubleSpinBox_electron_voltage.setDecimals(0)
        self.doubleSpinBox_electron_voltage.setMaximum(100000000000.0)
        self.doubleSpinBox_electron_voltage.setObjectName("doubleSpinBox_electron_voltage")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_voltage, 22, 1, 1, 2)
        self.label_configuration_ip_address = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_ip_address.setFont(font)
        self.label_configuration_ip_address.setObjectName("label_configuration_ip_address")
        self.gridLayout_2.addWidget(self.label_configuration_ip_address, 7, 0, 1, 1)
        self.lineEdit_electron_detector_type = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_electron_detector_type.setFont(font)
        self.lineEdit_electron_detector_type.setObjectName("lineEdit_electron_detector_type")
        self.gridLayout_2.addWidget(self.lineEdit_electron_detector_type, 28, 1, 1, 2)
        self.checkBox_subsystems_manipulator_rotation = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_manipulator_rotation.setFont(font)
        self.checkBox_subsystems_manipulator_rotation.setObjectName("checkBox_subsystems_manipulator_rotation")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_manipulator_rotation, 59, 1, 1, 1)
        self.lineEdit_configuration_path = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_configuration_path.setFont(font)
        self.lineEdit_configuration_path.setObjectName("lineEdit_configuration_path")
        self.gridLayout_2.addWidget(self.lineEdit_configuration_path, 5, 1, 1, 2)
        self.label_stage_rotation_reference = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_rotation_reference.setFont(font)
        self.label_stage_rotation_reference.setObjectName("label_stage_rotation_reference")
        self.gridLayout_2.addWidget(self.label_stage_rotation_reference, 11, 0, 1, 1)
        self.label_ion_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_ion_header.setFont(font)
        self.label_ion_header.setObjectName("label_ion_header")
        self.gridLayout_2.addWidget(self.label_ion_header, 29, 0, 1, 3)
        self.label_configuration_manufacturer = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_configuration_manufacturer.setFont(font)
        self.label_configuration_manufacturer.setObjectName("label_configuration_manufacturer")
        self.gridLayout_2.addWidget(self.label_configuration_manufacturer, 6, 0, 1, 1)
        self.label_imaging_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_imaging_header.setFont(font)
        self.label_imaging_header.setObjectName("label_imaging_header")
        self.gridLayout_2.addWidget(self.label_imaging_header, 40, 0, 1, 3)
        self.comboBox_configuration_manufacturer = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.comboBox_configuration_manufacturer.setFont(font)
        self.comboBox_configuration_manufacturer.setObjectName("comboBox_configuration_manufacturer")
        self.gridLayout_2.addWidget(self.comboBox_configuration_manufacturer, 6, 1, 1, 2)
        self.checkBox_subsystems_gis_sputter_coater_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_gis_sputter_coater_enabled.setFont(font)
        self.checkBox_subsystems_gis_sputter_coater_enabled.setObjectName("checkBox_subsystems_gis_sputter_coater_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_gis_sputter_coater_enabled, 60, 2, 1, 1)
        self.doubleSpinBox_milling_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_current.setFont(font)
        self.doubleSpinBox_milling_current.setMaximum(100000000.0)
        self.doubleSpinBox_milling_current.setObjectName("doubleSpinBox_milling_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_current, 49, 1, 1, 2)
        self.spinBox_ion_column_tilt = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_ion_column_tilt.setFont(font)
        self.spinBox_ion_column_tilt.setMaximum(180)
        self.spinBox_ion_column_tilt.setObjectName("spinBox_ion_column_tilt")
        self.gridLayout_2.addWidget(self.spinBox_ion_column_tilt, 31, 1, 1, 2)
        self.label_electron_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_current.setFont(font)
        self.label_electron_current.setObjectName("label_electron_current")
        self.gridLayout_2.addWidget(self.label_electron_current, 23, 0, 1, 1)
        self.checkBox_subsystems_stage_rotation = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_stage_rotation.setFont(font)
        self.checkBox_subsystems_stage_rotation.setObjectName("checkBox_subsystems_stage_rotation")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_stage_rotation, 10, 1, 1, 1)
        self.checkBox_subsystems_gis_multichem_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_gis_multichem_enabled.setFont(font)
        self.checkBox_subsystems_gis_multichem_enabled.setObjectName("checkBox_subsystems_gis_multichem_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_gis_multichem_enabled, 60, 1, 1, 1)
        self.label_milling_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_voltage.setFont(font)
        self.label_milling_voltage.setObjectName("label_milling_voltage")
        self.gridLayout_2.addWidget(self.label_milling_voltage, 48, 0, 1, 1)
        self.checkBox_subsystems_ion_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_ion_enabled.setFont(font)
        self.checkBox_subsystems_ion_enabled.setObjectName("checkBox_subsystems_ion_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_ion_enabled, 30, 0, 1, 1)
        self.checkBox_subsystems_manipulator_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_manipulator_enabled.setFont(font)
        self.checkBox_subsystems_manipulator_enabled.setObjectName("checkBox_subsystems_manipulator_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_manipulator_enabled, 59, 0, 1, 1)
        self.label_stage_shuttle_pre_tilt = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_shuttle_pre_tilt.setFont(font)
        self.label_stage_shuttle_pre_tilt.setObjectName("label_stage_shuttle_pre_tilt")
        self.gridLayout_2.addWidget(self.label_stage_shuttle_pre_tilt, 13, 0, 1, 1)
        self.doubleSpinBox_electron_eucentric_height = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_eucentric_height.setFont(font)
        self.doubleSpinBox_electron_eucentric_height.setObjectName("doubleSpinBox_electron_eucentric_height")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_eucentric_height, 21, 1, 1, 2)
        self.label_electron_eucentric_height = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_eucentric_height.setFont(font)
        self.label_electron_eucentric_height.setObjectName("label_electron_eucentric_height")
        self.gridLayout_2.addWidget(self.label_electron_eucentric_height, 21, 0, 1, 1)
        self.label_stage_manipulator_height_limit = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_stage_manipulator_height_limit.setFont(font)
        self.label_stage_manipulator_height_limit.setObjectName("label_stage_manipulator_height_limit")
        self.gridLayout_2.addWidget(self.label_stage_manipulator_height_limit, 14, 0, 1, 1)
        self.checkBox_subsystems_gis_enabled = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_gis_enabled.setFont(font)
        self.checkBox_subsystems_gis_enabled.setObjectName("checkBox_subsystems_gis_enabled")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_gis_enabled, 60, 0, 1, 1)
        self.label_electron_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_voltage.setFont(font)
        self.label_electron_voltage.setObjectName("label_electron_voltage")
        self.gridLayout_2.addWidget(self.label_electron_voltage, 22, 0, 1, 1)
        self.label_ion_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_resolution.setFont(font)
        self.label_ion_resolution.setObjectName("label_ion_resolution")
        self.gridLayout_2.addWidget(self.label_ion_resolution, 35, 0, 1, 1)
        self.doubleSpinBox_milling_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_milling_dwell_time.setFont(font)
        self.doubleSpinBox_milling_dwell_time.setMaximum(1000000.0)
        self.doubleSpinBox_milling_dwell_time.setObjectName("doubleSpinBox_milling_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_milling_dwell_time, 51, 1, 1, 2)
        self.label_ion_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_current.setFont(font)
        self.label_ion_current.setObjectName("label_ion_current")
        self.gridLayout_2.addWidget(self.label_ion_current, 34, 0, 1, 1)
        self.label_ion_column_tilt = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_column_tilt.setFont(font)
        self.label_ion_column_tilt.setObjectName("label_ion_column_tilt")
        self.gridLayout_2.addWidget(self.label_ion_column_tilt, 31, 0, 1, 1)
        self.lineEdit_milling_preset = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_milling_preset.setFont(font)
        self.lineEdit_milling_preset.setObjectName("lineEdit_milling_preset")
        self.gridLayout_2.addWidget(self.lineEdit_milling_preset, 54, 1, 1, 2)
        self.lineEdit_ion_detector_mode = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_ion_detector_mode.setFont(font)
        self.lineEdit_ion_detector_mode.setObjectName("lineEdit_ion_detector_mode")
        self.gridLayout_2.addWidget(self.lineEdit_ion_detector_mode, 38, 1, 1, 2)
        self.label_ion_eucentric_height = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_eucentric_height.setFont(font)
        self.label_ion_eucentric_height.setObjectName("label_ion_eucentric_height")
        self.gridLayout_2.addWidget(self.label_ion_eucentric_height, 32, 0, 1, 1)
        self.doubleSpinBox_imaging_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_imaging_dwell_time.setFont(font)
        self.doubleSpinBox_imaging_dwell_time.setMaximum(1000000.0)
        self.doubleSpinBox_imaging_dwell_time.setObjectName("doubleSpinBox_imaging_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_imaging_dwell_time, 44, 1, 1, 2)
        self.comboBox_imaging_beam_type = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.comboBox_imaging_beam_type.setFont(font)
        self.comboBox_imaging_beam_type.setObjectName("comboBox_imaging_beam_type")
        self.gridLayout_2.addWidget(self.comboBox_imaging_beam_type, 41, 1, 1, 2)
        self.doubleSpinBox_imaging_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_imaging_current.setFont(font)
        self.doubleSpinBox_imaging_current.setMaximum(10000000.0)
        self.doubleSpinBox_imaging_current.setObjectName("doubleSpinBox_imaging_current")
        self.gridLayout_2.addWidget(self.doubleSpinBox_imaging_current, 45, 1, 1, 2)
        self.label_milling_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_current.setFont(font)
        self.label_milling_current.setObjectName("label_milling_current")
        self.gridLayout_2.addWidget(self.label_milling_current, 49, 0, 1, 1)
        self.label_subsystems_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_subsystems_header.setFont(font)
        self.label_subsystems_header.setObjectName("label_subsystems_header")
        self.gridLayout_2.addWidget(self.label_subsystems_header, 55, 0, 1, 3)
        self.label_stage_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.label_stage_header.setFont(font)
        self.label_stage_header.setObjectName("label_stage_header")
        self.gridLayout_2.addWidget(self.label_stage_header, 9, 0, 1, 3)
        self.spinBox_imaging_resolution_x = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_imaging_resolution_x.setFont(font)
        self.spinBox_imaging_resolution_x.setMaximum(100000)
        self.spinBox_imaging_resolution_x.setObjectName("spinBox_imaging_resolution_x")
        self.gridLayout_2.addWidget(self.spinBox_imaging_resolution_x, 42, 1, 1, 1)
        self.label_electron_detector_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_detector_type.setFont(font)
        self.label_electron_detector_type.setObjectName("label_electron_detector_type")
        self.gridLayout_2.addWidget(self.label_electron_detector_type, 28, 0, 1, 1)
        self.label_electron_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_electron_resolution.setFont(font)
        self.label_electron_resolution.setObjectName("label_electron_resolution")
        self.gridLayout_2.addWidget(self.label_electron_resolution, 24, 0, 1, 1)
        self.lineEdit_configuration_name = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lineEdit_configuration_name.setFont(font)
        self.lineEdit_configuration_name.setObjectName("lineEdit_configuration_name")
        self.gridLayout_2.addWidget(self.lineEdit_configuration_name, 4, 1, 1, 2)
        self.spinBox_imaging_resolution_y = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_imaging_resolution_y.setFont(font)
        self.spinBox_imaging_resolution_y.setMaximum(100000)
        self.spinBox_imaging_resolution_y.setObjectName("spinBox_imaging_resolution_y")
        self.gridLayout_2.addWidget(self.spinBox_imaging_resolution_y, 42, 2, 1, 1)
        self.label_milling_preset = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_milling_preset.setFont(font)
        self.label_milling_preset.setObjectName("label_milling_preset")
        self.gridLayout_2.addWidget(self.label_milling_preset, 54, 0, 1, 1)
        self.doubleSpinBox_stage_manipulator_height_limit = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_stage_manipulator_height_limit.setFont(font)
        self.doubleSpinBox_stage_manipulator_height_limit.setObjectName("doubleSpinBox_stage_manipulator_height_limit")
        self.gridLayout_2.addWidget(self.doubleSpinBox_stage_manipulator_height_limit, 14, 1, 1, 2)
        self.checkBox_subsystems_manipulator_tilt = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_subsystems_manipulator_tilt.setFont(font)
        self.checkBox_subsystems_manipulator_tilt.setObjectName("checkBox_subsystems_manipulator_tilt")
        self.gridLayout_2.addWidget(self.checkBox_subsystems_manipulator_tilt, 59, 2, 1, 1)
        self.label_ion_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_ion_dwell_time.setFont(font)
        self.label_ion_dwell_time.setObjectName("label_ion_dwell_time")
        self.gridLayout_2.addWidget(self.label_ion_dwell_time, 37, 0, 1, 1)
        self.doubleSpinBox_ion_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_dwell_time.setFont(font)
        self.doubleSpinBox_ion_dwell_time.setMaximum(1000000.0)
        self.doubleSpinBox_ion_dwell_time.setObjectName("doubleSpinBox_ion_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_dwell_time, 37, 1, 1, 2)
        self.doubleSpinBox_electron_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_dwell_time.setFont(font)
        self.doubleSpinBox_electron_dwell_time.setMaximum(10000000.0)
        self.doubleSpinBox_electron_dwell_time.setObjectName("doubleSpinBox_electron_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_dwell_time, 26, 1, 1, 2)
        self.doubleSpinBox_electron_hfw = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_electron_hfw.setFont(font)
        self.doubleSpinBox_electron_hfw.setDecimals(0)
        self.doubleSpinBox_electron_hfw.setMaximum(100000.0)
        self.doubleSpinBox_electron_hfw.setObjectName("doubleSpinBox_electron_hfw")
        self.gridLayout_2.addWidget(self.doubleSpinBox_electron_hfw, 25, 1, 1, 2)
        self.doubleSpinBox_ion_hfw = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.doubleSpinBox_ion_hfw.setFont(font)
        self.doubleSpinBox_ion_hfw.setDecimals(0)
        self.doubleSpinBox_ion_hfw.setMaximum(100000.0)
        self.doubleSpinBox_ion_hfw.setObjectName("doubleSpinBox_ion_hfw")
        self.gridLayout_2.addWidget(self.doubleSpinBox_ion_hfw, 36, 1, 1, 2)
        self.spinBox_electron_resolution_x = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_electron_resolution_x.setFont(font)
        self.spinBox_electron_resolution_x.setMaximum(100000)
        self.spinBox_electron_resolution_x.setObjectName("spinBox_electron_resolution_x")
        self.gridLayout_2.addWidget(self.spinBox_electron_resolution_x, 24, 1, 1, 1)
        self.spinBox_electron_resolution_y = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_electron_resolution_y.setFont(font)
        self.spinBox_electron_resolution_y.setMaximum(100000)
        self.spinBox_electron_resolution_y.setObjectName("spinBox_electron_resolution_y")
        self.gridLayout_2.addWidget(self.spinBox_electron_resolution_y, 24, 2, 1, 1)
        self.spinBox_ion_resolution_x = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_ion_resolution_x.setFont(font)
        self.spinBox_ion_resolution_x.setMaximum(100000000)
        self.spinBox_ion_resolution_x.setObjectName("spinBox_ion_resolution_x")
        self.gridLayout_2.addWidget(self.spinBox_ion_resolution_x, 35, 1, 1, 1)
        self.spinBox_ion_resolution_y = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.spinBox_ion_resolution_y.setFont(font)
        self.spinBox_ion_resolution_y.setMaximum(100000000)
        self.spinBox_ion_resolution_y.setObjectName("spinBox_ion_resolution_y")
        self.gridLayout_2.addWidget(self.spinBox_ion_resolution_y, 35, 2, 1, 1)
        self.checkBox_show_advanced_settings = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.checkBox_show_advanced_settings.setFont(font)
        self.checkBox_show_advanced_settings.setObjectName("checkBox_show_advanced_settings")
        self.gridLayout_2.addWidget(self.checkBox_show_advanced_settings, 0, 1, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 2, 0, 1, 2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_electron_detector_mode.setText(_translate("Form", "Detector Mode"))
        self.label_configuration_path.setText(_translate("Form", "Path"))
        self.label_ion_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_configuration_name.setText(_translate("Form", "Name"))
        self.label_milling_header.setText(_translate("Form", "Milling"))
        self.checkBox_subsystems_ion_plasma.setText(_translate("Form", "Ion Beam Plasma"))
        self.label_milling_spot_size.setText(_translate("Form", "Spot Size (nm)"))
        self.checkBox_subsystems_stage_tilt.setText(_translate("Form", "Stage Tilt"))
        self.checkBox_imaging_autogamma.setText(_translate("Form", "AutoGamma"))
        self.label_imaging_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.label_ion_detector_mode.setText(_translate("Form", "Detector Mode"))
        self.label_imaging_resolution.setText(_translate("Form", "Resolution"))
        self.checkBox_subsystems_stage_enabled.setText(_translate("Form", "Stage Enabled"))
        self.label_milling_rate.setText(_translate("Form", "Rate (um3/s)"))
        self.label_electron_column_tilt.setText(_translate("Form", "Column Tilt (deg)"))
        self.label_electron_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_ion_voltage.setText(_translate("Form", "Voltage (V)"))
        self.label_milling_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.label_electron_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.label_electron_header.setText(_translate("Form", "Electron Beam"))
        self.label_configuration_header.setText(_translate("Form", "Core"))
        self.label_ion_detector_type.setText(_translate("Form", "Detector Type"))
        self.label_imaging_current.setText(_translate("Form", "Imaging Current (nA)"))
        self.checkBox_imaging_autosave.setText(_translate("Form", "AutoSave"))
        self.label_imaging_beam_type.setText(_translate("Form", "Beam Type"))
        self.checkBox_imaging_autocontrast.setText(_translate("Form", "AutoContrast"))
        self.checkBox_subsystems_electron_enabled.setText(_translate("Form", "Electron Beam Enabled"))
        self.label_stage_rotation_180.setText(_translate("Form", "Rotation 180 (deg)"))
        self.label_imaging_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_configuration_ip_address.setText(_translate("Form", "IP Address"))
        self.checkBox_subsystems_manipulator_rotation.setText(_translate("Form", "Manipulator Rotation"))
        self.label_stage_rotation_reference.setText(_translate("Form", "Rotation Reference (deg)"))
        self.label_ion_header.setText(_translate("Form", "Ion Beam"))
        self.label_configuration_manufacturer.setText(_translate("Form", "Manufacturer"))
        self.label_imaging_header.setText(_translate("Form", "Imaging"))
        self.checkBox_subsystems_gis_sputter_coater_enabled.setText(_translate("Form", "Sputter Coat Enabled"))
        self.label_electron_current.setText(_translate("Form", "Current (nA)"))
        self.checkBox_subsystems_stage_rotation.setText(_translate("Form", "Stage Rotation"))
        self.checkBox_subsystems_gis_multichem_enabled.setText(_translate("Form", "MultiChem Enabled"))
        self.label_milling_voltage.setText(_translate("Form", "Milling Voltage (V)"))
        self.checkBox_subsystems_ion_enabled.setText(_translate("Form", "Ion Beam Enabled"))
        self.checkBox_subsystems_manipulator_enabled.setText(_translate("Form", "Manipulator Enabled"))
        self.label_stage_shuttle_pre_tilt.setText(_translate("Form", "Shuttle Pre-Tilt (deg)"))
        self.label_electron_eucentric_height.setText(_translate("Form", "Eucentric Height (mm)"))
        self.label_stage_manipulator_height_limit.setText(_translate("Form", "Manipulator Height Limit (mm)"))
        self.checkBox_subsystems_gis_enabled.setText(_translate("Form", "GIS Enabled"))
        self.label_electron_voltage.setText(_translate("Form", "Voltage (V)"))
        self.label_ion_resolution.setText(_translate("Form", "Resolution"))
        self.label_ion_current.setText(_translate("Form", "Current (nA)"))
        self.label_ion_column_tilt.setText(_translate("Form", "Column Tilt (deg)"))
        self.label_ion_eucentric_height.setText(_translate("Form", "Eucentric Height (mm)"))
        self.label_milling_current.setText(_translate("Form", "Milling Current (nA)"))
        self.label_subsystems_header.setText(_translate("Form", "Subsystems"))
        self.label_stage_header.setText(_translate("Form", "Stage"))
        self.label_electron_detector_type.setText(_translate("Form", "Detector Type"))
        self.label_electron_resolution.setText(_translate("Form", "Resolution"))
        self.label_milling_preset.setText(_translate("Form", "Preset "))
        self.checkBox_subsystems_manipulator_tilt.setText(_translate("Form", "Manipulator Tilt"))
        self.label_ion_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.checkBox_show_advanced_settings.setText(_translate("Form", "Show Advanced Settings"))
