# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemCryoDepositionWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(400, 300)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.label_title = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 2)
        self.label_sputter_stage_position = QtWidgets.QLabel(Dialog)
        self.label_sputter_stage_position.setObjectName("label_sputter_stage_position")
        self.gridLayout.addWidget(self.label_sputter_stage_position, 1, 0, 1, 1)
        self.comboBox_sputter_stage_position = QtWidgets.QComboBox(Dialog)
        self.comboBox_sputter_stage_position.setObjectName("comboBox_sputter_stage_position")
        self.gridLayout.addWidget(self.comboBox_sputter_stage_position, 1, 1, 1, 1)
        self.label_sputter_application_file = QtWidgets.QLabel(Dialog)
        self.label_sputter_application_file.setObjectName("label_sputter_application_file")
        self.gridLayout.addWidget(self.label_sputter_application_file, 2, 0, 1, 1)
        self.lineEdit_sputter_application_file = QtWidgets.QLineEdit(Dialog)
        self.lineEdit_sputter_application_file.setObjectName("lineEdit_sputter_application_file")
        self.gridLayout.addWidget(self.lineEdit_sputter_application_file, 2, 1, 1, 1)
        self.label_sputter_gis_position = QtWidgets.QLabel(Dialog)
        self.label_sputter_gis_position.setObjectName("label_sputter_gis_position")
        self.gridLayout.addWidget(self.label_sputter_gis_position, 3, 0, 1, 1)
        self.lineEdit_sputter_gis_position = QtWidgets.QLineEdit(Dialog)
        self.lineEdit_sputter_gis_position.setObjectName("lineEdit_sputter_gis_position")
        self.gridLayout.addWidget(self.lineEdit_sputter_gis_position, 3, 1, 1, 1)
        self.label_sputter_gas = QtWidgets.QLabel(Dialog)
        self.label_sputter_gas.setObjectName("label_sputter_gas")
        self.gridLayout.addWidget(self.label_sputter_gas, 4, 0, 1, 1)
        self.lineEdit_sputter_gas = QtWidgets.QLineEdit(Dialog)
        self.lineEdit_sputter_gas.setObjectName("lineEdit_sputter_gas")
        self.gridLayout.addWidget(self.lineEdit_sputter_gas, 4, 1, 1, 1)
        self.label_sputter_hfw = QtWidgets.QLabel(Dialog)
        self.label_sputter_hfw.setObjectName("label_sputter_hfw")
        self.gridLayout.addWidget(self.label_sputter_hfw, 5, 0, 1, 1)
        self.doubleSpinBox_sputter_hfw = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_hfw.setMaximum(1000.0)
        self.doubleSpinBox_sputter_hfw.setProperty("value", 30.0)
        self.doubleSpinBox_sputter_hfw.setObjectName("doubleSpinBox_sputter_hfw")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_hfw, 5, 1, 1, 1)
        self.label_sputter_length = QtWidgets.QLabel(Dialog)
        self.label_sputter_length.setObjectName("label_sputter_length")
        self.gridLayout.addWidget(self.label_sputter_length, 6, 0, 1, 1)
        self.doubleSpinBox_sputter_length = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_length.setMaximum(1000.0)
        self.doubleSpinBox_sputter_length.setProperty("value", 7.5)
        self.doubleSpinBox_sputter_length.setObjectName("doubleSpinBox_sputter_length")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_length, 6, 1, 1, 1)
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 7, 0, 1, 1)
        self.doubleSpinBox_sputter_beam_current = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_beam_current.setMaximum(1000.0)
        self.doubleSpinBox_sputter_beam_current.setProperty("value", 10.0)
        self.doubleSpinBox_sputter_beam_current.setObjectName("doubleSpinBox_sputter_beam_current")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_beam_current, 7, 1, 1, 1)
        self.label_sputter_time = QtWidgets.QLabel(Dialog)
        self.label_sputter_time.setObjectName("label_sputter_time")
        self.gridLayout.addWidget(self.label_sputter_time, 8, 0, 1, 1)
        self.doubleSpinBox_sputter_time = QtWidgets.QDoubleSpinBox(Dialog)
        self.doubleSpinBox_sputter_time.setMaximum(1000.0)
        self.doubleSpinBox_sputter_time.setProperty("value", 30.0)
        self.doubleSpinBox_sputter_time.setObjectName("doubleSpinBox_sputter_time")
        self.gridLayout.addWidget(self.doubleSpinBox_sputter_time, 8, 1, 1, 1)
        self.pushButton_run_sputter = QtWidgets.QPushButton(Dialog)
        self.pushButton_run_sputter.setObjectName("pushButton_run_sputter")
        self.gridLayout.addWidget(self.pushButton_run_sputter, 9, 0, 1, 2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label_title.setText(_translate("Dialog", "Cryo Sputter"))
        self.label_sputter_stage_position.setText(_translate("Dialog", "Stage Position"))
        self.label_sputter_application_file.setText(_translate("Dialog", "Application File"))
        self.lineEdit_sputter_application_file.setText(_translate("Dialog", "cryo_Pt_dep"))
        self.label_sputter_gis_position.setText(_translate("Dialog", "GIS Position"))
        self.lineEdit_sputter_gis_position.setText(_translate("Dialog", "cryo"))
        self.label_sputter_gas.setText(_translate("Dialog", "Gas"))
        self.lineEdit_sputter_gas.setText(_translate("Dialog", "Pt cryo"))
        self.label_sputter_hfw.setText(_translate("Dialog", "HFW (um)"))
        self.label_sputter_length.setText(_translate("Dialog", "Length (um)"))
        self.label.setText(_translate("Dialog", "Current (nA)"))
        self.label_sputter_time.setText(_translate("Dialog", "Time (s)"))
        self.pushButton_run_sputter.setText(_translate("Dialog", "Run Sputtering"))
