# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemDetectionWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(365, 646)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.checkBox_show_info = QtWidgets.QCheckBox(Form)
        self.checkBox_show_info.setObjectName("checkBox_show_info")
        self.gridLayout.addWidget(self.checkBox_show_info, 14, 2, 1, 1)
        self.comboBox_feature_2 = QtWidgets.QComboBox(Form)
        self.comboBox_feature_2.setObjectName("comboBox_feature_2")
        self.gridLayout.addWidget(self.comboBox_feature_2, 12, 2, 1, 1)
        self.label_num_classes = QtWidgets.QLabel(Form)
        self.label_num_classes.setObjectName("label_num_classes")
        self.gridLayout.addWidget(self.label_num_classes, 9, 0, 1, 1)
        self.lineEdit_checkpoint = QtWidgets.QLineEdit(Form)
        self.lineEdit_checkpoint.setObjectName("lineEdit_checkpoint")
        self.gridLayout.addWidget(self.lineEdit_checkpoint, 8, 1, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 31, 0, 1, 3)
        self.label_instructions = QtWidgets.QLabel(Form)
        self.label_instructions.setWordWrap(True)
        self.label_instructions.setObjectName("label_instructions")
        self.gridLayout.addWidget(self.label_instructions, 29, 0, 1, 3)
        self.label_image_path_num = QtWidgets.QLabel(Form)
        self.label_image_path_num.setAlignment(QtCore.Qt.AlignCenter)
        self.label_image_path_num.setObjectName("label_image_path_num")
        self.gridLayout.addWidget(self.label_image_path_num, 26, 1, 1, 1)
        self.pushButton_previous_image = QtWidgets.QPushButton(Form)
        self.pushButton_previous_image.setObjectName("pushButton_previous_image")
        self.gridLayout.addWidget(self.pushButton_previous_image, 26, 0, 1, 1)
        self.label_title = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 3)
        self.spinBox_num_classes = QtWidgets.QSpinBox(Form)
        self.spinBox_num_classes.setObjectName("spinBox_num_classes")
        self.gridLayout.addWidget(self.spinBox_num_classes, 9, 1, 1, 2)
        self.label_header_features = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_header_features.setFont(font)
        self.label_header_features.setObjectName("label_header_features")
        self.gridLayout.addWidget(self.label_header_features, 11, 0, 1, 3)
        self.pushButton_run_feature_detection = QtWidgets.QPushButton(Form)
        self.pushButton_run_feature_detection.setObjectName("pushButton_run_feature_detection")
        self.gridLayout.addWidget(self.pushButton_run_feature_detection, 15, 0, 1, 3)
        self.checkBox_use_segmentation = QtWidgets.QCheckBox(Form)
        self.checkBox_use_segmentation.setChecked(True)
        self.checkBox_use_segmentation.setObjectName("checkBox_use_segmentation")
        self.gridLayout.addWidget(self.checkBox_use_segmentation, 1, 0, 1, 1)
        self.pushButton_load_images = QtWidgets.QPushButton(Form)
        self.pushButton_load_images.setObjectName("pushButton_load_images")
        self.gridLayout.addWidget(self.pushButton_load_images, 25, 0, 1, 3)
        self.label_encoder = QtWidgets.QLabel(Form)
        self.label_encoder.setObjectName("label_encoder")
        self.gridLayout.addWidget(self.label_encoder, 7, 0, 1, 1)
        self.label_info = QtWidgets.QLabel(Form)
        self.label_info.setMinimumSize(QtCore.QSize(0, 0))
        self.label_info.setText("")
        self.label_info.setObjectName("label_info")
        self.gridLayout.addWidget(self.label_info, 16, 0, 1, 3)
        self.checkBox_use_feature_detection = QtWidgets.QCheckBox(Form)
        self.checkBox_use_feature_detection.setChecked(True)
        self.checkBox_use_feature_detection.setObjectName("checkBox_use_feature_detection")
        self.gridLayout.addWidget(self.checkBox_use_feature_detection, 1, 1, 1, 1)
        self.pushButton_continue = QtWidgets.QPushButton(Form)
        self.pushButton_continue.setObjectName("pushButton_continue")
        self.gridLayout.addWidget(self.pushButton_continue, 30, 0, 1, 3)
        self.label_checkpoint = QtWidgets.QLabel(Form)
        self.label_checkpoint.setObjectName("label_checkpoint")
        self.gridLayout.addWidget(self.label_checkpoint, 8, 0, 1, 1)
        self.lineEdit_encoder = QtWidgets.QLineEdit(Form)
        self.lineEdit_encoder.setObjectName("lineEdit_encoder")
        self.gridLayout.addWidget(self.lineEdit_encoder, 7, 1, 1, 2)
        self.label_dataset = QtWidgets.QLabel(Form)
        self.label_dataset.setObjectName("label_dataset")
        self.gridLayout.addWidget(self.label_dataset, 24, 0, 1, 1)
        self.pushButton_next_image = QtWidgets.QPushButton(Form)
        self.pushButton_next_image.setObjectName("pushButton_next_image")
        self.gridLayout.addWidget(self.pushButton_next_image, 26, 2, 1, 1)
        self.label_header_model = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_header_model.setFont(font)
        self.label_header_model.setObjectName("label_header_model")
        self.gridLayout.addWidget(self.label_header_model, 2, 0, 1, 3)
        self.label_feature = QtWidgets.QLabel(Form)
        self.label_feature.setObjectName("label_feature")
        self.gridLayout.addWidget(self.label_feature, 12, 0, 1, 1)
        self.checkBox_use_evaluation = QtWidgets.QCheckBox(Form)
        self.checkBox_use_evaluation.setObjectName("checkBox_use_evaluation")
        self.gridLayout.addWidget(self.checkBox_use_evaluation, 1, 2, 1, 1)
        self.pushButton_load_model = QtWidgets.QPushButton(Form)
        self.pushButton_load_model.setObjectName("pushButton_load_model")
        self.gridLayout.addWidget(self.pushButton_load_model, 10, 0, 1, 3)
        self.label_header_evalulation = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_header_evalulation.setFont(font)
        self.label_header_evalulation.setObjectName("label_header_evalulation")
        self.gridLayout.addWidget(self.label_header_evalulation, 22, 0, 1, 3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 27, 0, 1, 3)
        self.lineEdit_image_path_folder = QtWidgets.QLineEdit(Form)
        self.lineEdit_image_path_folder.setObjectName("lineEdit_image_path_folder")
        self.gridLayout.addWidget(self.lineEdit_image_path_folder, 24, 1, 1, 2)
        self.checkBox_move_features = QtWidgets.QCheckBox(Form)
        self.checkBox_move_features.setObjectName("checkBox_move_features")
        self.gridLayout.addWidget(self.checkBox_move_features, 14, 1, 1, 1)
        self.comboBox_feature_1 = QtWidgets.QComboBox(Form)
        self.comboBox_feature_1.setObjectName("comboBox_feature_1")
        self.gridLayout.addWidget(self.comboBox_feature_1, 12, 1, 1, 1)
        self.checkBox_all_features = QtWidgets.QCheckBox(Form)
        self.checkBox_all_features.setObjectName("checkBox_all_features")
        self.gridLayout.addWidget(self.checkBox_all_features, 14, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.checkBox_use_segmentation, self.checkBox_use_feature_detection)
        Form.setTabOrder(self.checkBox_use_feature_detection, self.checkBox_use_evaluation)
        Form.setTabOrder(self.checkBox_use_evaluation, self.lineEdit_encoder)
        Form.setTabOrder(self.lineEdit_encoder, self.lineEdit_checkpoint)
        Form.setTabOrder(self.lineEdit_checkpoint, self.spinBox_num_classes)
        Form.setTabOrder(self.spinBox_num_classes, self.pushButton_load_model)
        Form.setTabOrder(self.pushButton_load_model, self.comboBox_feature_1)
        Form.setTabOrder(self.comboBox_feature_1, self.comboBox_feature_2)
        Form.setTabOrder(self.comboBox_feature_2, self.checkBox_all_features)
        Form.setTabOrder(self.checkBox_all_features, self.checkBox_move_features)
        Form.setTabOrder(self.checkBox_move_features, self.checkBox_show_info)
        Form.setTabOrder(self.checkBox_show_info, self.pushButton_run_feature_detection)
        Form.setTabOrder(self.pushButton_run_feature_detection, self.lineEdit_image_path_folder)
        Form.setTabOrder(self.lineEdit_image_path_folder, self.pushButton_load_images)
        Form.setTabOrder(self.pushButton_load_images, self.pushButton_previous_image)
        Form.setTabOrder(self.pushButton_previous_image, self.pushButton_next_image)
        Form.setTabOrder(self.pushButton_next_image, self.pushButton_continue)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.checkBox_show_info.setText(_translate("Form", "Show Info"))
        self.label_num_classes.setText(_translate("Form", "No. Classes"))
        self.label_instructions.setText(_translate("Form", "Drag to move the features, when finished press continue."))
        self.label_image_path_num.setText(_translate("Form", "TextLabel"))
        self.pushButton_previous_image.setText(_translate("Form", "Previous"))
        self.label_title.setText(_translate("Form", "Feature Detection"))
        self.label_header_features.setText(_translate("Form", "Feature Detection"))
        self.pushButton_run_feature_detection.setText(_translate("Form", "Run Feature Detection"))
        self.checkBox_use_segmentation.setText(_translate("Form", "Segmentation"))
        self.pushButton_load_images.setText(_translate("Form", "Load Images"))
        self.label_encoder.setText(_translate("Form", "Encoder"))
        self.checkBox_use_feature_detection.setText(_translate("Form", "Feature Detection"))
        self.pushButton_continue.setText(_translate("Form", "Continue"))
        self.label_checkpoint.setText(_translate("Form", "Checkpoint"))
        self.label_dataset.setText(_translate("Form", "Dataset"))
        self.pushButton_next_image.setText(_translate("Form", "Next"))
        self.label_header_model.setText(_translate("Form", "Segmentation Model"))
        self.label_feature.setText(_translate("Form", "Features"))
        self.checkBox_use_evaluation.setText(_translate("Form", "Evalulation"))
        self.pushButton_load_model.setText(_translate("Form", "Load Model"))
        self.label_header_evalulation.setText(_translate("Form", "Evaluation Mode"))
        self.checkBox_move_features.setText(_translate("Form", "Move Features"))
        self.checkBox_all_features.setText(_translate("Form", "All Features"))
