# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemFeatureDetectionUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(429, 648)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.comboBox_features = QtWidgets.QComboBox(self.centralwidget)
        self.comboBox_features.setObjectName("comboBox_features")
        self.gridLayout.addWidget(self.comboBox_features, 3, 0, 1, 2)
        self.label_header_features = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_header_features.setFont(font)
        self.label_header_features.setObjectName("label_header_features")
        self.gridLayout.addWidget(self.label_header_features, 0, 0, 1, 2)
        self.pushButton_edit_feature = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_edit_feature.setObjectName("pushButton_edit_feature")
        self.gridLayout.addWidget(self.pushButton_edit_feature, 2, 1, 1, 1)
        self.label_features_header = QtWidgets.QLabel(self.centralwidget)
        self.label_features_header.setObjectName("label_features_header")
        self.gridLayout.addWidget(self.label_features_header, 1, 0, 1, 2)
        self.pushButton_add_feature = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_add_feature.setObjectName("pushButton_add_feature")
        self.gridLayout.addWidget(self.pushButton_add_feature, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 7, 0, 1, 2)
        self.label_instructions = QtWidgets.QLabel(self.centralwidget)
        self.label_instructions.setWordWrap(True)
        self.label_instructions.setObjectName("label_instructions")
        self.gridLayout.addWidget(self.label_instructions, 6, 0, 1, 2)
        self.spinBox_data_progress = QtWidgets.QSpinBox(self.centralwidget)
        self.spinBox_data_progress.setAlignment(QtCore.Qt.AlignCenter)
        self.spinBox_data_progress.setMaximum(999999999)
        self.spinBox_data_progress.setObjectName("spinBox_data_progress")
        self.gridLayout.addWidget(self.spinBox_data_progress, 5, 1, 1, 1)
        self.label_data_progress = QtWidgets.QLabel(self.centralwidget)
        self.label_data_progress.setObjectName("label_data_progress")
        self.gridLayout.addWidget(self.label_data_progress, 5, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 429, 22))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionLoad_Image_Directory = QtWidgets.QAction(MainWindow)
        self.actionLoad_Image_Directory.setObjectName("actionLoad_Image_Directory")
        self.actionLoad_CSV = QtWidgets.QAction(MainWindow)
        self.actionLoad_CSV.setObjectName("actionLoad_CSV")
        self.menuFile.addAction(self.actionLoad_Image_Directory)
        self.menuFile.addAction(self.actionLoad_CSV)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label_header_features.setText(_translate("MainWindow", "OpenFIBSEM Keypoint Labelling"))
        self.pushButton_edit_feature.setText(_translate("MainWindow", "Edit"))
        self.label_features_header.setText(_translate("MainWindow", "Features"))
        self.pushButton_add_feature.setText(_translate("MainWindow", "Add"))
        self.label_instructions.setText(_translate("MainWindow", "Instructions"))
        self.label_data_progress.setText(_translate("MainWindow", "Current Image (Total: N)"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.actionLoad_Image_Directory.setText(_translate("MainWindow", "Load Image Directory"))
        self.actionLoad_CSV.setText(_translate("MainWindow", "Load CSV"))
