# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CurrentAlignmentWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_BeamAlignment(object):
    def setupUi(self, BeamAlignment):
        BeamAlignment.setObjectName("BeamAlignment")
        BeamAlignment.resize(398, 380)
        self.gridLayout = QtWidgets.QGridLayout(BeamAlignment)
        self.gridLayout.setObjectName("gridLayout")
        self.label_3 = QtWidgets.QLabel(BeamAlignment)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 4, 0, 1, 1)
        self.pushButton_align_beam = QtWidgets.QPushButton(BeamAlignment)
        self.pushButton_align_beam.setObjectName("pushButton_align_beam")
        self.gridLayout.addWidget(self.pushButton_align_beam, 6, 0, 1, 2)
        self.checkBox_overlay = QtWidgets.QCheckBox(BeamAlignment)
        self.checkBox_overlay.setObjectName("checkBox_overlay")
        self.gridLayout.addWidget(self.checkBox_overlay, 2, 1, 1, 1)
        self.listdone = QtWidgets.QPlainTextEdit(BeamAlignment)
        self.listdone.setObjectName("listdone")
        self.gridLayout.addWidget(self.listdone, 7, 0, 1, 2)
        self.label = QtWidgets.QLabel(BeamAlignment)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.comboBox_aligned_current = QtWidgets.QComboBox(BeamAlignment)
        self.comboBox_aligned_current.setObjectName("comboBox_aligned_current")
        self.gridLayout.addWidget(self.comboBox_aligned_current, 4, 1, 1, 1)
        self.comboBox_ref_current = QtWidgets.QComboBox(BeamAlignment)
        self.comboBox_ref_current.setObjectName("comboBox_ref_current")
        self.gridLayout.addWidget(self.comboBox_ref_current, 3, 1, 1, 1)
        self.pushButton_take_images = QtWidgets.QPushButton(BeamAlignment)
        self.pushButton_take_images.setObjectName("pushButton_take_images")
        self.gridLayout.addWidget(self.pushButton_take_images, 5, 0, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 0, 1, 2)
        self.label_2 = QtWidgets.QLabel(BeamAlignment)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 8, 0, 1, 2)

        self.retranslateUi(BeamAlignment)
        QtCore.QMetaObject.connectSlotsByName(BeamAlignment)
        BeamAlignment.setTabOrder(self.checkBox_overlay, self.comboBox_ref_current)
        BeamAlignment.setTabOrder(self.comboBox_ref_current, self.comboBox_aligned_current)
        BeamAlignment.setTabOrder(self.comboBox_aligned_current, self.pushButton_take_images)
        BeamAlignment.setTabOrder(self.pushButton_take_images, self.pushButton_align_beam)
        BeamAlignment.setTabOrder(self.pushButton_align_beam, self.listdone)

    def retranslateUi(self, BeamAlignment):
        _translate = QtCore.QCoreApplication.translate
        BeamAlignment.setWindowTitle(_translate("BeamAlignment", "Beam alignment"))
        self.label_3.setText(_translate("BeamAlignment", "Aligned Current (Right):"))
        self.pushButton_align_beam.setText(_translate("BeamAlignment", "Align Beam"))
        self.checkBox_overlay.setText(_translate("BeamAlignment", "Overlay Images"))
        self.label.setText(_translate("BeamAlignment", "FIB Current alignment"))
        self.pushButton_take_images.setText(_translate("BeamAlignment", "Take Images"))
        self.label_2.setText(_translate("BeamAlignment", "Reference Current (Left):"))
