# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemMultiChemWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(256, 240)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.label_hfw = QtWidgets.QLabel(Form)
        self.label_hfw.setObjectName("label_hfw")
        self.gridLayout.addWidget(self.label_hfw, 5, 0, 1, 1)
        self.label_gas = QtWidgets.QLabel(Form)
        self.label_gas.setObjectName("label_gas")
        self.gridLayout.addWidget(self.label_gas, 4, 0, 1, 1)
        self.label_title = QtWidgets.QLabel(Form)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 2)
        self.comboBox_beam_type = QtWidgets.QComboBox(Form)
        self.comboBox_beam_type.setObjectName("comboBox_beam_type")
        self.gridLayout.addWidget(self.comboBox_beam_type, 1, 1, 1, 1)
        self.pushButton_run_sputtering = QtWidgets.QPushButton(Form)
        self.pushButton_run_sputtering.setObjectName("pushButton_run_sputtering")
        self.gridLayout.addWidget(self.pushButton_run_sputtering, 8, 0, 1, 2)
        self.label_position = QtWidgets.QLabel(Form)
        self.label_position.setObjectName("label_position")
        self.gridLayout.addWidget(self.label_position, 3, 0, 1, 1)
        self.label_application_file = QtWidgets.QLabel(Form)
        self.label_application_file.setObjectName("label_application_file")
        self.gridLayout.addWidget(self.label_application_file, 2, 0, 1, 1)
        self.label_beam_type = QtWidgets.QLabel(Form)
        self.label_beam_type.setObjectName("label_beam_type")
        self.gridLayout.addWidget(self.label_beam_type, 1, 0, 1, 1)
        self.doubleSpinBox_hfw = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_hfw.setMaximum(2700.0)
        self.doubleSpinBox_hfw.setProperty("value", 150.0)
        self.doubleSpinBox_hfw.setObjectName("doubleSpinBox_hfw")
        self.gridLayout.addWidget(self.doubleSpinBox_hfw, 5, 1, 1, 1)
        self.doubleSpinBox_length = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_length.setObjectName("doubleSpinBox_length")
        self.gridLayout.addWidget(self.doubleSpinBox_length, 6, 1, 1, 1)
        self.label_length = QtWidgets.QLabel(Form)
        self.label_length.setObjectName("label_length")
        self.gridLayout.addWidget(self.label_length, 6, 0, 1, 1)
        self.comboBox_position = QtWidgets.QComboBox(Form)
        self.comboBox_position.setObjectName("comboBox_position")
        self.gridLayout.addWidget(self.comboBox_position, 3, 1, 1, 1)
        self.comboBox_application_file = QtWidgets.QComboBox(Form)
        self.comboBox_application_file.setObjectName("comboBox_application_file")
        self.gridLayout.addWidget(self.comboBox_application_file, 2, 1, 1, 1)
        self.comboBox_gas = QtWidgets.QComboBox(Form)
        self.comboBox_gas.setObjectName("comboBox_gas")
        self.gridLayout.addWidget(self.comboBox_gas, 4, 1, 1, 1)
        self.label_time = QtWidgets.QLabel(Form)
        self.label_time.setObjectName("label_time")
        self.gridLayout.addWidget(self.label_time, 7, 0, 1, 1)
        self.doubleSpinBox_time = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_time.setProperty("value", 30.0)
        self.doubleSpinBox_time.setObjectName("doubleSpinBox_time")
        self.gridLayout.addWidget(self.doubleSpinBox_time, 7, 1, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_gas.setText(_translate("Form", "Gas"))
        self.label_title.setText(_translate("Form", "MultiChem"))
        self.pushButton_run_sputtering.setText(_translate("Form", "Run Sputtering"))
        self.label_position.setText(_translate("Form", "Position"))
        self.label_application_file.setText(_translate("Form", "Application File"))
        self.label_beam_type.setText(_translate("Form", "Beam Type"))
        self.label_length.setText(_translate("Form", "Length (um)"))
        self.label_time.setText(_translate("Form", "Time (sec)"))
