# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemSegmentationModelWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(456, 352)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.checkpoint_seg_button = QtWidgets.QToolButton(Form)
        self.checkpoint_seg_button.setObjectName("checkpoint_seg_button")
        self.gridLayout.addWidget(self.checkpoint_seg_button, 2, 2, 1, 1)
        self.label_model_type = QtWidgets.QLabel(Form)
        self.label_model_type.setObjectName("label_model_type")
        self.gridLayout.addWidget(self.label_model_type, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 4, 0, 1, 2)
        self.label_header_model = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_header_model.setFont(font)
        self.label_header_model.setObjectName("label_header_model")
        self.gridLayout.addWidget(self.label_header_model, 0, 0, 1, 2)
        self.label_checkpoint = QtWidgets.QLabel(Form)
        self.label_checkpoint.setObjectName("label_checkpoint")
        self.gridLayout.addWidget(self.label_checkpoint, 2, 0, 1, 1)
        self.comboBox_model_type = QtWidgets.QComboBox(Form)
        self.comboBox_model_type.setObjectName("comboBox_model_type")
        self.gridLayout.addWidget(self.comboBox_model_type, 1, 1, 1, 1)
        self.lineEdit_checkpoint = QtWidgets.QLineEdit(Form)
        self.lineEdit_checkpoint.setObjectName("lineEdit_checkpoint")
        self.gridLayout.addWidget(self.lineEdit_checkpoint, 2, 1, 1, 1)
        self.pushButton_load_model = QtWidgets.QPushButton(Form)
        self.pushButton_load_model.setObjectName("pushButton_load_model")
        self.gridLayout.addWidget(self.pushButton_load_model, 3, 0, 1, 2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.checkpoint_seg_button.setText(_translate("Form", "..."))
        self.label_model_type.setText(_translate("Form", "Model"))
        self.label_header_model.setText(_translate("Form", "Segmentation Model"))
        self.label_checkpoint.setText(_translate("Form", "Checkpoint"))
        self.pushButton_load_model.setText(_translate("Form", "Load Model"))
