import numpy as np
MICRON_TO_METRE = 1e-6
METRE_TO_MICRON = 1.0 / MICRON_TO_METRE

MILLIMETRE_TO_METRE = 1e-3
METRE_TO_MILLIMETRE = 1.0 / MILLIMETRE_TO_METRE

SI_TO_KILO = 1e-3
KILO_TO_SI =  1/ SI_TO_KILO

SI_TO_MILLI = 1e3
MILLI_TO_SI = 1 / SI_TO_MILLI

SI_TO_MICRO = 1e6
MICRO_TO_SI = 1 / SI_TO_MICRO

SI_TO_NANO = 1e9
NANO_TO_SI = 1 / SI_TO_NANO

SI_TO_PICO = 1e12
PICO_TO_SI = 1 / SI_TO_PICO

RADIANS_TO_DEGREES = 180.0 / np.pi

DEGREES_TO_RADIANS = np.pi / 180.0

TO_PERCENTAGES = 100.0

FROM_PERCENTAGES = 1.0 / TO_PERCENTAGES