# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'image_viewer.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(603, 833)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 569, 845))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_30 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_30.setObjectName("label_30")
        self.gridLayout_2.addWidget(self.label_30, 33, 0, 1, 1)
        self.label_fibsem_version = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_fibsem_version.setText("")
        self.label_fibsem_version.setObjectName("label_fibsem_version")
        self.gridLayout_2.addWidget(self.label_fibsem_version, 14, 1, 1, 2)
        self.line_5 = QtWidgets.QFrame(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.line_5.setFont(font)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.gridLayout_2.addWidget(self.line_5, 36, 0, 1, 3)
        self.line_2 = QtWidgets.QFrame(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.line_2.setFont(font)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_2.addWidget(self.line_2, 21, 0, 1, 3)
        self.label_15 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.label_15.setFont(font)
        self.label_15.setObjectName("label_15")
        self.gridLayout_2.addWidget(self.label_15, 16, 0, 1, 1)
        self.label_name = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_name.setFont(font)
        self.label_name.setText("")
        self.label_name.setObjectName("label_name")
        self.gridLayout_2.addWidget(self.label_name, 1, 1, 1, 2)
        self.label_detector_mode = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_mode.setText("")
        self.label_detector_mode.setObjectName("label_detector_mode")
        self.gridLayout_2.addWidget(self.label_detector_mode, 19, 1, 1, 2)
        self.line_6 = QtWidgets.QFrame(self.scrollAreaWidgetContents)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.gridLayout_2.addWidget(self.line_6, 42, 0, 1, 3)
        self.label_11 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 11, 0, 1, 1)
        self.label_method = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_method.setText("")
        self.label_method.setObjectName("label_method")
        self.gridLayout_2.addWidget(self.label_method, 12, 1, 1, 2)
        self.label_20 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.gridLayout_2.addWidget(self.label_20, 22, 0, 1, 1)
        self.label_detector_brightness = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_brightness.setText("")
        self.label_detector_brightness.setObjectName("label_detector_brightness")
        self.gridLayout_2.addWidget(self.label_detector_brightness, 17, 1, 1, 2)
        self.label_14 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setItalic(False)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 14, 0, 1, 1)
        self.label_21 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_21.setObjectName("label_21")
        self.gridLayout_2.addWidget(self.label_21, 23, 0, 1, 1)
        self.label_timestamp = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_timestamp.setFont(font)
        self.label_timestamp.setText("")
        self.label_timestamp.setObjectName("label_timestamp")
        self.gridLayout_2.addWidget(self.label_timestamp, 6, 1, 1, 2)
        self.label_application = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_application.setText("")
        self.label_application.setObjectName("label_application")
        self.gridLayout_2.addWidget(self.label_application, 13, 1, 1, 1)
        self.label_serial_no = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_serial_no.setText("")
        self.label_serial_no.setObjectName("label_serial_no")
        self.gridLayout_2.addWidget(self.label_serial_no, 34, 1, 1, 2)
        self.label_19 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_19.setObjectName("label_19")
        self.gridLayout_2.addWidget(self.label_19, 20, 0, 1, 1)
        self.label_shift = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_shift.setText("")
        self.label_shift.setObjectName("label_shift")
        self.gridLayout_2.addWidget(self.label_shift, 28, 1, 1, 2)
        self.label_app_version = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_app_version.setText("")
        self.label_app_version.setObjectName("label_app_version")
        self.gridLayout_2.addWidget(self.label_app_version, 13, 2, 1, 1)
        self.line_4 = QtWidgets.QFrame(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.line_4.setFont(font)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.gridLayout_2.addWidget(self.line_4, 9, 0, 1, 3)
        self.label_29 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_29.setObjectName("label_29")
        self.gridLayout_2.addWidget(self.label_29, 32, 0, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_16.setObjectName("label_16")
        self.gridLayout_2.addWidget(self.label_16, 17, 0, 1, 1)
        self.line_3 = QtWidgets.QFrame(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.line_3.setFont(font)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.gridLayout_2.addWidget(self.line_3, 30, 0, 1, 3)
        self.label_31 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_31.setObjectName("label_31")
        self.gridLayout_2.addWidget(self.label_31, 34, 0, 1, 1)
        self.label_metadata_version = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_metadata_version.setFont(font)
        self.label_metadata_version.setText("")
        self.label_metadata_version.setObjectName("label_metadata_version")
        self.gridLayout_2.addWidget(self.label_metadata_version, 44, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 10, 0, 1, 1)
        self.line = QtWidgets.QFrame(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.line.setFont(font)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 15, 0, 1, 3)
        self.label_2 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 3, 0, 1, 1)
        self.label_25 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_25.setObjectName("label_25")
        self.gridLayout_2.addWidget(self.label_25, 29, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 43, 0, 1, 3)
        self.label_6 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 5, 0, 1, 1)
        self.label_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_dwell_time.setText("")
        self.label_dwell_time.setObjectName("label_dwell_time")
        self.gridLayout_2.addWidget(self.label_dwell_time, 26, 1, 1, 2)
        self.label_software_version = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_software_version.setText("")
        self.label_software_version.setObjectName("label_software_version")
        self.gridLayout_2.addWidget(self.label_software_version, 35, 1, 1, 2)
        self.label_27 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_27.setObjectName("label_27")
        self.gridLayout_2.addWidget(self.label_27, 25, 0, 1, 1)
        self.label_manufacturer = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_manufacturer.setText("")
        self.label_manufacturer.setObjectName("label_manufacturer")
        self.gridLayout_2.addWidget(self.label_manufacturer, 32, 1, 1, 2)
        self.label_23 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_23.setObjectName("label_23")
        self.gridLayout_2.addWidget(self.label_23, 27, 0, 1, 1)
        self.label_28 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.label_28.setFont(font)
        self.label_28.setObjectName("label_28")
        self.gridLayout_2.addWidget(self.label_28, 31, 0, 1, 1)
        self.label_organisation = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_organisation.setText("")
        self.label_organisation.setObjectName("label_organisation")
        self.gridLayout_2.addWidget(self.label_organisation, 39, 1, 1, 2)
        self.label_beam_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_beam_current.setText("")
        self.label_beam_current.setObjectName("label_beam_current")
        self.gridLayout_2.addWidget(self.label_beam_current, 23, 1, 1, 2)
        self.label_model = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_model.setText("")
        self.label_model.setObjectName("label_model")
        self.gridLayout_2.addWidget(self.label_model, 33, 1, 1, 2)
        self.label_stigmation = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_stigmation.setText("")
        self.label_stigmation.setObjectName("label_stigmation")
        self.gridLayout_2.addWidget(self.label_stigmation, 29, 1, 1, 2)
        self.label_working_distance = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_working_distance.setText("")
        self.label_working_distance.setObjectName("label_working_distance")
        self.gridLayout_2.addWidget(self.label_working_distance, 25, 1, 1, 2)
        self.label_32 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_32.setObjectName("label_32")
        self.gridLayout_2.addWidget(self.label_32, 35, 0, 1, 1)
        self.label_userpc = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_userpc.setText("")
        self.label_userpc.setObjectName("label_userpc")
        self.gridLayout_2.addWidget(self.label_userpc, 40, 1, 1, 2)
        self.label_18 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_18.setObjectName("label_18")
        self.gridLayout_2.addWidget(self.label_18, 19, 0, 1, 1)
        self.label_ID = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_ID.setText("")
        self.label_ID.setObjectName("label_ID")
        self.gridLayout_2.addWidget(self.label_ID, 11, 1, 1, 2)
        self.label_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_voltage.setText("")
        self.label_voltage.setObjectName("label_voltage")
        self.gridLayout_2.addWidget(self.label_voltage, 24, 1, 1, 2)
        self.label_37 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_37.setObjectName("label_37")
        self.gridLayout_2.addWidget(self.label_37, 41, 0, 1, 1)
        self.label_beamType = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_beamType.setFont(font)
        self.label_beamType.setText("")
        self.label_beamType.setObjectName("label_beamType")
        self.gridLayout_2.addWidget(self.label_beamType, 3, 1, 1, 2)
        self.label_33 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.label_33.setFont(font)
        self.label_33.setObjectName("label_33")
        self.gridLayout_2.addWidget(self.label_33, 37, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 44, 0, 1, 1)
        self.label_26 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_26.setObjectName("label_26")
        self.gridLayout_2.addWidget(self.label_26, 24, 0, 1, 1)
        self.label_detector_contrast = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_contrast.setText("")
        self.label_detector_contrast.setObjectName("label_detector_contrast")
        self.gridLayout_2.addWidget(self.label_detector_contrast, 18, 1, 1, 2)
        self.label_username = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_username.setText("")
        self.label_username.setObjectName("label_username")
        self.gridLayout_2.addWidget(self.label_username, 38, 1, 1, 2)
        self.label_12 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 12, 0, 1, 1)
        self.label_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_hfw.setFont(font)
        self.label_hfw.setText("")
        self.label_hfw.setObjectName("label_hfw")
        self.gridLayout_2.addWidget(self.label_hfw, 5, 1, 1, 2)
        self.label_34 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_34.setObjectName("label_34")
        self.gridLayout_2.addWidget(self.label_34, 38, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 4, 0, 1, 1)
        self.label_detector_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_type.setText("")
        self.label_detector_type.setObjectName("label_detector_type")
        self.gridLayout_2.addWidget(self.label_detector_type, 20, 1, 1, 2)
        self.label_17 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_17.setObjectName("label_17")
        self.gridLayout_2.addWidget(self.label_17, 18, 0, 1, 1)
        self.label_38 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.label_38.setFont(font)
        self.label_38.setObjectName("label_38")
        self.gridLayout_2.addWidget(self.label_38, 7, 0, 1, 1)
        self.label_35 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_35.setObjectName("label_35")
        self.gridLayout_2.addWidget(self.label_35, 39, 0, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_24.setObjectName("label_24")
        self.gridLayout_2.addWidget(self.label_24, 28, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 1, 0, 1, 1)
        self.label_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_resolution.setFont(font)
        self.label_resolution.setText("")
        self.label_resolution.setObjectName("label_resolution")
        self.gridLayout_2.addWidget(self.label_resolution, 4, 1, 1, 2)
        self.label_13 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 13, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(False)
        font.setWeight(50)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 6, 0, 1, 1)
        self.label_36 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_36.setObjectName("label_36")
        self.gridLayout_2.addWidget(self.label_36, 40, 0, 1, 1)
        self.label_scan_rotation = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_scan_rotation.setText("")
        self.label_scan_rotation.setObjectName("label_scan_rotation")
        self.gridLayout_2.addWidget(self.label_scan_rotation, 27, 1, 1, 2)
        self.label_22 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_22.setObjectName("label_22")
        self.gridLayout_2.addWidget(self.label_22, 26, 0, 1, 1)
        self.label_email = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_email.setText("")
        self.label_email.setObjectName("label_email")
        self.gridLayout_2.addWidget(self.label_email, 41, 1, 1, 2)
        self.label_position = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_position.setText("")
        self.label_position.setObjectName("label_position")
        self.gridLayout_2.addWidget(self.label_position, 8, 0, 1, 3)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 4, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 2, 0, 1, 1)
        self.pushButton_export = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_export.setObjectName("pushButton_export")
        self.gridLayout.addWidget(self.pushButton_export, 3, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 603, 20))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label_30.setText(_translate("MainWindow", "Model"))
        self.label_15.setText(_translate("MainWindow", "Detector Settings"))
        self.label_11.setText(_translate("MainWindow", "Experiment ID"))
        self.label_20.setText(_translate("MainWindow", "Beam Settings"))
        self.label_14.setText(_translate("MainWindow", "OpenFIBSEM Version"))
        self.label_21.setText(_translate("MainWindow", "Beam Current"))
        self.label_19.setText(_translate("MainWindow", "Type"))
        self.label_29.setText(_translate("MainWindow", "Manufacturer"))
        self.label_16.setText(_translate("MainWindow", "Brightness"))
        self.label_31.setText(_translate("MainWindow", "Serial No."))
        self.label_10.setText(_translate("MainWindow", "Experiment"))
        self.label_2.setText(_translate("MainWindow", "Beam Type"))
        self.label_25.setText(_translate("MainWindow", "Stigmation"))
        self.label_6.setText(_translate("MainWindow", "Horizontal Field Width (um)"))
        self.label_27.setText(_translate("MainWindow", "Working Distance"))
        self.label_23.setText(_translate("MainWindow", "Scan Rotation"))
        self.label_28.setText(_translate("MainWindow", "System"))
        self.label_32.setText(_translate("MainWindow", "Software Version"))
        self.label_18.setText(_translate("MainWindow", "Mode"))
        self.label_37.setText(_translate("MainWindow", "Email"))
        self.label_33.setText(_translate("MainWindow", "User"))
        self.label_8.setText(_translate("MainWindow", "Image Metadata Version"))
        self.label_26.setText(_translate("MainWindow", "Voltage"))
        self.label_12.setText(_translate("MainWindow", "Method"))
        self.label_34.setText(_translate("MainWindow", "Name"))
        self.label_5.setText(_translate("MainWindow", "Resolution"))
        self.label_17.setText(_translate("MainWindow", "Contrast"))
        self.label_38.setText(_translate("MainWindow", "Stage Position"))
        self.label_35.setText(_translate("MainWindow", "Organisation"))
        self.label_24.setText(_translate("MainWindow", "Shift"))
        self.label_7.setText(_translate("MainWindow", "Filename"))
        self.label_13.setText(_translate("MainWindow", "Application (Version)"))
        self.label_9.setText(_translate("MainWindow", "Time Stamp "))
        self.label_36.setText(_translate("MainWindow", "Computer"))
        self.label_22.setText(_translate("MainWindow", "Dwell Time"))
        self.label_3.setText(_translate("MainWindow", "Drag and drop images into the viewer"))
        self.label.setText(_translate("MainWindow", "OpenFIBSEM Image Viewer"))
        self.label_4.setText(_translate("MainWindow", "Drag and drop of folders is currently unsupported"))
        self.pushButton_export.setText(_translate("MainWindow", "Export Image with Metadata"))
