# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemLabellingUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(400, 469)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.comboBox_model_class_index = QtWidgets.QComboBox(Dialog)
        self.comboBox_model_class_index.setObjectName("comboBox_model_class_index")
        self.gridLayout.addWidget(self.comboBox_model_class_index, 4, 1, 1, 1)
        self.checkBox_model_assist = QtWidgets.QCheckBox(Dialog)
        self.checkBox_model_assist.setObjectName("checkBox_model_assist")
        self.gridLayout.addWidget(self.checkBox_model_assist, 2, 0, 1, 1)
        self.label_model_info = QtWidgets.QLabel(Dialog)
        self.label_model_info.setObjectName("label_model_info")
        self.gridLayout.addWidget(self.label_model_info, 2, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 13, 0, 1, 2)
        self.label_instructions = QtWidgets.QLabel(Dialog)
        self.label_instructions.setWordWrap(True)
        self.label_instructions.setObjectName("label_instructions")
        self.gridLayout.addWidget(self.label_instructions, 11, 0, 1, 2)
        self.pushButton_model_clear = QtWidgets.QPushButton(Dialog)
        self.pushButton_model_clear.setObjectName("pushButton_model_clear")
        self.gridLayout.addWidget(self.pushButton_model_clear, 5, 1, 1, 1)
        self.pushButton_model_confirm = QtWidgets.QPushButton(Dialog)
        self.pushButton_model_confirm.setObjectName("pushButton_model_confirm")
        self.gridLayout.addWidget(self.pushButton_model_confirm, 5, 0, 1, 1)
        self.label_title = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 2)
        self.tabWidget = QtWidgets.QTabWidget(Dialog)
        self.tabWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_data_file_ext = QtWidgets.QLabel(self.tab)
        self.label_data_file_ext.setObjectName("label_data_file_ext")
        self.gridLayout_2.addWidget(self.label_data_file_ext, 3, 0, 1, 1)
        self.label_labels_path = QtWidgets.QLabel(self.tab)
        self.label_labels_path.setObjectName("label_labels_path")
        self.gridLayout_2.addWidget(self.label_labels_path, 1, 0, 1, 1)
        self.checkBox_save_rgb = QtWidgets.QCheckBox(self.tab)
        self.checkBox_save_rgb.setChecked(False)
        self.checkBox_save_rgb.setObjectName("checkBox_save_rgb")
        self.gridLayout_2.addWidget(self.checkBox_save_rgb, 6, 1, 1, 1)
        self.pushButton_labels_path = QtWidgets.QToolButton(self.tab)
        self.pushButton_labels_path.setObjectName("pushButton_labels_path")
        self.gridLayout_2.addWidget(self.pushButton_labels_path, 1, 2, 1, 1)
        self.lineEdit_labels_path = QtWidgets.QLineEdit(self.tab)
        self.lineEdit_labels_path.setObjectName("lineEdit_labels_path")
        self.gridLayout_2.addWidget(self.lineEdit_labels_path, 1, 1, 1, 1)
        self.pushButton_load_data = QtWidgets.QPushButton(self.tab)
        self.pushButton_load_data.setObjectName("pushButton_load_data")
        self.gridLayout_2.addWidget(self.pushButton_load_data, 4, 0, 1, 3)
        self.label_data_path = QtWidgets.QLabel(self.tab)
        self.label_data_path.setObjectName("label_data_path")
        self.gridLayout_2.addWidget(self.label_data_path, 0, 0, 1, 1)
        self.lineEdit_data_path = QtWidgets.QLineEdit(self.tab)
        self.lineEdit_data_path.setObjectName("lineEdit_data_path")
        self.gridLayout_2.addWidget(self.lineEdit_data_path, 0, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 5, 0, 1, 3)
        self.pushButton_data_path = QtWidgets.QToolButton(self.tab)
        self.pushButton_data_path.setObjectName("pushButton_data_path")
        self.gridLayout_2.addWidget(self.pushButton_data_path, 0, 2, 1, 1)
        self.comboBox_data_file_ext = QtWidgets.QComboBox(self.tab)
        self.comboBox_data_file_ext.setObjectName("comboBox_data_file_ext")
        self.gridLayout_2.addWidget(self.comboBox_data_file_ext, 3, 1, 1, 2)
        self.checkBox_autosave = QtWidgets.QCheckBox(self.tab)
        self.checkBox_autosave.setChecked(True)
        self.checkBox_autosave.setObjectName("checkBox_autosave")
        self.gridLayout_2.addWidget(self.checkBox_autosave, 6, 0, 1, 1)
        self.label_data_config = QtWidgets.QLabel(self.tab)
        self.label_data_config.setObjectName("label_data_config")
        self.gridLayout_2.addWidget(self.label_data_config, 2, 0, 1, 1)
        self.lineEdit_data_config = QtWidgets.QLineEdit(self.tab)
        self.lineEdit_data_config.setObjectName("lineEdit_data_config")
        self.gridLayout_2.addWidget(self.lineEdit_data_config, 2, 1, 1, 1)
        self.pushButton_data_config = QtWidgets.QToolButton(self.tab)
        self.pushButton_data_config.setObjectName("pushButton_data_config")
        self.gridLayout_2.addWidget(self.pushButton_data_config, 2, 2, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.gridLayout.addWidget(self.tabWidget, 1, 0, 1, 2)
        self.label_model_class_index = QtWidgets.QLabel(Dialog)
        self.label_model_class_index.setObjectName("label_model_class_index")
        self.gridLayout.addWidget(self.label_model_class_index, 4, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.tabWidget, self.lineEdit_data_path)
        Dialog.setTabOrder(self.lineEdit_data_path, self.pushButton_data_path)
        Dialog.setTabOrder(self.pushButton_data_path, self.lineEdit_labels_path)
        Dialog.setTabOrder(self.lineEdit_labels_path, self.pushButton_labels_path)
        Dialog.setTabOrder(self.pushButton_labels_path, self.pushButton_load_data)
        Dialog.setTabOrder(self.pushButton_load_data, self.checkBox_model_assist)
        Dialog.setTabOrder(self.checkBox_model_assist, self.comboBox_model_class_index)
        Dialog.setTabOrder(self.comboBox_model_class_index, self.pushButton_model_confirm)
        Dialog.setTabOrder(self.pushButton_model_confirm, self.pushButton_model_clear)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.checkBox_model_assist.setText(_translate("Dialog", "Model Assisted"))
        self.label_model_info.setText(_translate("Dialog", "No Model"))
        self.label_instructions.setText(_translate("Dialog", "Instructions"))
        self.pushButton_model_clear.setText(_translate("Dialog", "Clear"))
        self.pushButton_model_confirm.setText(_translate("Dialog", "Confirm"))
        self.label_title.setText(_translate("Dialog", "OpenFIBSEM Labelling"))
        self.label_data_file_ext.setText(_translate("Dialog", "File Extension"))
        self.label_labels_path.setText(_translate("Dialog", "Label Path"))
        self.checkBox_save_rgb.setText(_translate("Dialog", "Save RGB Masks"))
        self.pushButton_labels_path.setText(_translate("Dialog", "..."))
        self.pushButton_load_data.setText(_translate("Dialog", "Load Data"))
        self.label_data_path.setText(_translate("Dialog", "Image Path"))
        self.pushButton_data_path.setText(_translate("Dialog", "..."))
        self.checkBox_autosave.setText(_translate("Dialog", "AutoSave"))
        self.label_data_config.setText(_translate("Dialog", "Class Config"))
        self.pushButton_data_config.setText(_translate("Dialog", "..."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("Dialog", "Data"))
        self.label_model_class_index.setText(_translate("Dialog", "Class Map"))
