# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemEmbeddedDetectionWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(375, 465)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.label_labelling_class_index = QtWidgets.QLabel(Form)
        self.label_labelling_class_index.setObjectName("label_labelling_class_index")
        self.gridLayout.addWidget(self.label_labelling_class_index, 8, 0, 1, 1)
        self.label_labelling_instructions = QtWidgets.QLabel(Form)
        self.label_labelling_instructions.setWordWrap(True)
        self.label_labelling_instructions.setObjectName("label_labelling_instructions")
        self.gridLayout.addWidget(self.label_labelling_instructions, 10, 0, 1, 2)
        self.label_info = QtWidgets.QLabel(Form)
        self.label_info.setMinimumSize(QtCore.QSize(0, 0))
        self.label_info.setText("")
        self.label_info.setWordWrap(True)
        self.label_info.setObjectName("label_info")
        self.gridLayout.addWidget(self.label_info, 2, 0, 1, 2)
        self.pushButton_labelling_cancel = QtWidgets.QPushButton(Form)
        self.pushButton_labelling_cancel.setObjectName("pushButton_labelling_cancel")
        self.gridLayout.addWidget(self.pushButton_labelling_cancel, 9, 1, 1, 1)
        self.comboBox_labelling_class_index = QtWidgets.QComboBox(Form)
        self.comboBox_labelling_class_index.setObjectName("comboBox_labelling_class_index")
        self.gridLayout.addWidget(self.comboBox_labelling_class_index, 8, 1, 1, 1)
        self.label_instructions = QtWidgets.QLabel(Form)
        self.label_instructions.setWordWrap(True)
        self.label_instructions.setObjectName("label_instructions")
        self.gridLayout.addWidget(self.label_instructions, 3, 0, 1, 2)
        self.pushButton_enable_labelling = QtWidgets.QPushButton(Form)
        self.pushButton_enable_labelling.setObjectName("pushButton_enable_labelling")
        self.gridLayout.addWidget(self.pushButton_enable_labelling, 6, 0, 1, 2)
        self.label_model = QtWidgets.QLabel(Form)
        self.label_model.setObjectName("label_model")
        self.gridLayout.addWidget(self.label_model, 1, 0, 1, 2)
        self.pushButton_labelling_confirm = QtWidgets.QPushButton(Form)
        self.pushButton_labelling_confirm.setObjectName("pushButton_labelling_confirm")
        self.gridLayout.addWidget(self.pushButton_labelling_confirm, 9, 0, 1, 1)
        self.label_labelling_model = QtWidgets.QLabel(Form)
        self.label_labelling_model.setText("")
        self.label_labelling_model.setObjectName("label_labelling_model")
        self.gridLayout.addWidget(self.label_labelling_model, 7, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 11, 0, 1, 2)
        self.pushButton_continue = QtWidgets.QPushButton(Form)
        self.pushButton_continue.setObjectName("pushButton_continue")
        self.gridLayout.addWidget(self.pushButton_continue, 4, 0, 1, 2)
        self.checkBox_labelling_model_assist = QtWidgets.QCheckBox(Form)
        self.checkBox_labelling_model_assist.setObjectName("checkBox_labelling_model_assist")
        self.gridLayout.addWidget(self.checkBox_labelling_model_assist, 7, 0, 1, 1)
        self.label_labelling_header = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_labelling_header.setFont(font)
        self.label_labelling_header.setObjectName("label_labelling_header")
        self.gridLayout.addWidget(self.label_labelling_header, 5, 0, 1, 2)
        self.label_title = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setWeight(75)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_labelling_class_index.setText(_translate("Form", "Class Index"))
        self.label_labelling_instructions.setText(_translate("Form", "Use the paint tools to label the images."))
        self.pushButton_labelling_cancel.setText(_translate("Form", "Cancel"))
        self.label_instructions.setText(_translate("Form", "Drag to move the features, when finished press confirm."))
        self.pushButton_enable_labelling.setText(_translate("Form", "Enable Labelling"))
        self.label_model.setText(_translate("Form", "Model:"))
        self.pushButton_labelling_confirm.setText(_translate("Form", "Confirm"))
        self.pushButton_continue.setText(_translate("Form", "Confirm"))
        self.checkBox_labelling_model_assist.setText(_translate("Form", "Segment Anything Assist"))
        self.label_labelling_header.setText(_translate("Form", "Labelling"))
        self.label_title.setText(_translate("Form", "Feature Detection"))
