# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemMovementWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(477, 698)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.label_movement_header = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_movement_header.setFont(font)
        self.label_movement_header.setObjectName("label_movement_header")
        self.gridLayout.addWidget(self.label_movement_header, 0, 0, 1, 2)
        self.pushButton_save_position = QtWidgets.QPushButton(Form)
        self.pushButton_save_position.setStyleSheet("color: white; background-color: rgb(85, 170, 0)")
        self.pushButton_save_position.setObjectName("pushButton_save_position")
        self.gridLayout.addWidget(self.pushButton_save_position, 15, 0, 1, 1)
        self.comboBox_positions = QtWidgets.QComboBox(Form)
        self.comboBox_positions.setObjectName("comboBox_positions")
        self.gridLayout.addWidget(self.comboBox_positions, 16, 1, 1, 1)
        self.label_movement_stage_z = QtWidgets.QLabel(Form)
        self.label_movement_stage_z.setObjectName("label_movement_stage_z")
        self.gridLayout.addWidget(self.label_movement_stage_z, 3, 0, 1, 1)
        self.pushButton_refresh_stage_position_data = QtWidgets.QPushButton(Form)
        self.pushButton_refresh_stage_position_data.setObjectName("pushButton_refresh_stage_position_data")
        self.gridLayout.addWidget(self.pushButton_refresh_stage_position_data, 6, 0, 1, 1)
        self.pushButton_move_flat_ion = QtWidgets.QPushButton(Form)
        self.pushButton_move_flat_ion.setObjectName("pushButton_move_flat_ion")
        self.gridLayout.addWidget(self.pushButton_move_flat_ion, 7, 1, 1, 1)
        self.label_positions_header = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_positions_header.setFont(font)
        self.label_positions_header.setObjectName("label_positions_header")
        self.gridLayout.addWidget(self.label_positions_header, 13, 0, 1, 2)
        self.pushButton_go_to = QtWidgets.QPushButton(Form)
        self.pushButton_go_to.setObjectName("pushButton_go_to")
        self.gridLayout.addWidget(self.pushButton_go_to, 18, 0, 1, 1)
        self.label_saved_positions = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_saved_positions.setFont(font)
        self.label_saved_positions.setObjectName("label_saved_positions")
        self.gridLayout.addWidget(self.label_saved_positions, 16, 0, 1, 1)
        self.label_movement_stage_x = QtWidgets.QLabel(Form)
        self.label_movement_stage_x.setObjectName("label_movement_stage_x")
        self.gridLayout.addWidget(self.label_movement_stage_x, 1, 0, 1, 1)
        self.pushButton_move_flat_electron = QtWidgets.QPushButton(Form)
        self.pushButton_move_flat_electron.setObjectName("pushButton_move_flat_electron")
        self.gridLayout.addWidget(self.pushButton_move_flat_electron, 7, 0, 1, 1)
        self.label_movement_instructions = QtWidgets.QLabel(Form)
        self.label_movement_instructions.setMinimumSize(QtCore.QSize(0, 0))
        self.label_movement_instructions.setWordWrap(True)
        self.label_movement_instructions.setObjectName("label_movement_instructions")
        self.gridLayout.addWidget(self.label_movement_instructions, 11, 0, 1, 2)
        self.label_minimap = QtWidgets.QLabel(Form)
        self.label_minimap.setText("")
        self.label_minimap.setObjectName("label_minimap")
        self.gridLayout.addWidget(self.label_minimap, 24, 0, 1, 2)
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 23, 0, 1, 1)
        self.lineEdit_position_name = QtWidgets.QLineEdit(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_position_name.sizePolicy().hasHeightForWidth())
        self.lineEdit_position_name.setSizePolicy(sizePolicy)
        self.lineEdit_position_name.setObjectName("lineEdit_position_name")
        self.gridLayout.addWidget(self.lineEdit_position_name, 19, 1, 1, 1)
        self.label_current_position = QtWidgets.QLabel(Form)
        self.label_current_position.setText("")
        self.label_current_position.setObjectName("label_current_position")
        self.gridLayout.addWidget(self.label_current_position, 18, 1, 1, 1)
        self.pushButton_import = QtWidgets.QPushButton(Form)
        self.pushButton_import.setObjectName("pushButton_import")
        self.gridLayout.addWidget(self.pushButton_import, 22, 0, 1, 1)
        self.checkBox_movement_acquire_electron = QtWidgets.QCheckBox(Form)
        self.checkBox_movement_acquire_electron.setChecked(True)
        self.checkBox_movement_acquire_electron.setObjectName("checkBox_movement_acquire_electron")
        self.gridLayout.addWidget(self.checkBox_movement_acquire_electron, 10, 0, 1, 1)
        self.doubleSpinBox_movement_stage_rotation = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_movement_stage_rotation.setMinimum(-360.0)
        self.doubleSpinBox_movement_stage_rotation.setMaximum(360.0)
        self.doubleSpinBox_movement_stage_rotation.setObjectName("doubleSpinBox_movement_stage_rotation")
        self.gridLayout.addWidget(self.doubleSpinBox_movement_stage_rotation, 4, 1, 1, 1)
        self.label_movement_stage_y = QtWidgets.QLabel(Form)
        self.label_movement_stage_y.setObjectName("label_movement_stage_y")
        self.gridLayout.addWidget(self.label_movement_stage_y, 2, 0, 1, 1)
        self.pushButton_load_image_minimap = QtWidgets.QPushButton(Form)
        self.pushButton_load_image_minimap.setObjectName("pushButton_load_image_minimap")
        self.gridLayout.addWidget(self.pushButton_load_image_minimap, 23, 1, 1, 1)
        self.pushButton_export = QtWidgets.QPushButton(Form)
        self.pushButton_export.setObjectName("pushButton_export")
        self.gridLayout.addWidget(self.pushButton_export, 22, 1, 1, 1)
        self.pushButton_remove_position = QtWidgets.QPushButton(Form)
        self.pushButton_remove_position.setStyleSheet("color: white; background-color: rgb(170, 0, 0)")
        self.pushButton_remove_position.setObjectName("pushButton_remove_position")
        self.gridLayout.addWidget(self.pushButton_remove_position, 15, 1, 1, 1)
        self.doubleSpinBox_movement_stage_y = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_movement_stage_y.setDecimals(5)
        self.doubleSpinBox_movement_stage_y.setMinimum(-1e+20)
        self.doubleSpinBox_movement_stage_y.setMaximum(1e+25)
        self.doubleSpinBox_movement_stage_y.setObjectName("doubleSpinBox_movement_stage_y")
        self.gridLayout.addWidget(self.doubleSpinBox_movement_stage_y, 2, 1, 1, 1)
        self.label_positions_header_info = QtWidgets.QLabel(Form)
        self.label_positions_header_info.setObjectName("label_positions_header_info")
        self.gridLayout.addWidget(self.label_positions_header_info, 14, 0, 1, 2)
        self.label_movement_stage_tilt = QtWidgets.QLabel(Form)
        self.label_movement_stage_tilt.setObjectName("label_movement_stage_tilt")
        self.gridLayout.addWidget(self.label_movement_stage_tilt, 5, 0, 1, 1)
        self.pushButton_update_position = QtWidgets.QPushButton(Form)
        self.pushButton_update_position.setObjectName("pushButton_update_position")
        self.gridLayout.addWidget(self.pushButton_update_position, 19, 0, 1, 1)
        self.doubleSpinBox_movement_stage_x = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_movement_stage_x.setDecimals(5)
        self.doubleSpinBox_movement_stage_x.setMinimum(-10000000000.0)
        self.doubleSpinBox_movement_stage_x.setMaximum(1e+17)
        self.doubleSpinBox_movement_stage_x.setObjectName("doubleSpinBox_movement_stage_x")
        self.gridLayout.addWidget(self.doubleSpinBox_movement_stage_x, 1, 1, 1, 1)
        self.label_movement_stage_rotation = QtWidgets.QLabel(Form)
        self.label_movement_stage_rotation.setObjectName("label_movement_stage_rotation")
        self.gridLayout.addWidget(self.label_movement_stage_rotation, 4, 0, 1, 1)
        self.label_movement_options_header = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_movement_options_header.setFont(font)
        self.label_movement_options_header.setObjectName("label_movement_options_header")
        self.gridLayout.addWidget(self.label_movement_options_header, 8, 0, 1, 2)
        self.checkBox_movement_acquire_ion = QtWidgets.QCheckBox(Form)
        self.checkBox_movement_acquire_ion.setChecked(True)
        self.checkBox_movement_acquire_ion.setObjectName("checkBox_movement_acquire_ion")
        self.gridLayout.addWidget(self.checkBox_movement_acquire_ion, 10, 1, 1, 1)
        self.pushButton_move = QtWidgets.QPushButton(Form)
        self.pushButton_move.setObjectName("pushButton_move")
        self.gridLayout.addWidget(self.pushButton_move, 6, 1, 1, 1)
        self.doubleSpinBox_movement_stage_z = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_movement_stage_z.setDecimals(5)
        self.doubleSpinBox_movement_stage_z.setMinimum(-1e+17)
        self.doubleSpinBox_movement_stage_z.setMaximum(1e+23)
        self.doubleSpinBox_movement_stage_z.setObjectName("doubleSpinBox_movement_stage_z")
        self.gridLayout.addWidget(self.doubleSpinBox_movement_stage_z, 3, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 25, 0, 1, 2)
        self.doubleSpinBox_movement_stage_tilt = QtWidgets.QDoubleSpinBox(Form)
        self.doubleSpinBox_movement_stage_tilt.setObjectName("doubleSpinBox_movement_stage_tilt")
        self.gridLayout.addWidget(self.doubleSpinBox_movement_stage_tilt, 5, 1, 1, 1)
        self.label_movement_images = QtWidgets.QLabel(Form)
        self.label_movement_images.setObjectName("label_movement_images")
        self.gridLayout.addWidget(self.label_movement_images, 9, 0, 1, 2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_movement_header.setText(_translate("Form", "Stage Movement"))
        self.pushButton_save_position.setText(_translate("Form", "Add Position"))
        self.label_movement_stage_z.setText(_translate("Form", "Z Coordinate (mm)"))
        self.pushButton_refresh_stage_position_data.setText(_translate("Form", "Refresh Stage Position Data"))
        self.pushButton_move_flat_ion.setText(_translate("Form", "Move Flat to ION Beam"))
        self.label_positions_header.setText(_translate("Form", "Stage Positions"))
        self.pushButton_go_to.setText(_translate("Form", "Go To Position"))
        self.label_saved_positions.setText(_translate("Form", "Saved Positions"))
        self.label_movement_stage_x.setText(_translate("Form", "X Coordinate (mm)"))
        self.pushButton_move_flat_electron.setText(_translate("Form", "Move Flat to ELECTRON Beam"))
        self.label_movement_instructions.setText(_translate("Form", "Double click to move. Press continue when complete."))
        self.label_4.setText(_translate("Form", "Please load grid image to display minimap "))
        self.pushButton_import.setText(_translate("Form", "Import Positions"))
        self.checkBox_movement_acquire_electron.setText(_translate("Form", "Electron Beam"))
        self.label_movement_stage_y.setText(_translate("Form", "Y Coordinate (mm)"))
        self.pushButton_load_image_minimap.setText(_translate("Form", "Load Minimap Image"))
        self.pushButton_export.setText(_translate("Form", "Export Positions"))
        self.pushButton_remove_position.setText(_translate("Form", "Remove Position"))
        self.label_positions_header_info.setText(_translate("Form", "All positions in mm and degrees"))
        self.label_movement_stage_tilt.setText(_translate("Form", "Tilt (deg)"))
        self.pushButton_update_position.setText(_translate("Form", "Update Position"))
        self.label_movement_stage_rotation.setText(_translate("Form", "Rotation (deg)"))
        self.label_movement_options_header.setText(_translate("Form", "Movement Options"))
        self.checkBox_movement_acquire_ion.setText(_translate("Form", "Ion Beam"))
        self.pushButton_move.setText(_translate("Form", "Move to Position"))
        self.label_movement_images.setText(_translate("Form", "Acquire images after moving:"))
