# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemPositionsWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(591, 505)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.comboBox_positions = QtWidgets.QComboBox(Form)
        self.comboBox_positions.setObjectName("comboBox_positions")
        self.gridLayout.addWidget(self.comboBox_positions, 2, 1, 1, 2)
        self.label = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.pushButton_remove_position = QtWidgets.QPushButton(Form)
        self.pushButton_remove_position.setObjectName("pushButton_remove_position")
        self.gridLayout.addWidget(self.pushButton_remove_position, 5, 0, 1, 1)
        self.pushButton_save_position = QtWidgets.QPushButton(Form)
        self.pushButton_save_position.setObjectName("pushButton_save_position")
        self.gridLayout.addWidget(self.pushButton_save_position, 4, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.lineEdit_position_name = QtWidgets.QLineEdit(Form)
        self.lineEdit_position_name.setObjectName("lineEdit_position_name")
        self.gridLayout.addWidget(self.lineEdit_position_name, 4, 1, 1, 2)
        self.pushButton_export = QtWidgets.QPushButton(Form)
        self.pushButton_export.setObjectName("pushButton_export")
        self.gridLayout.addWidget(self.pushButton_export, 6, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 9, 1, 1, 1)
        self.pushButton_import = QtWidgets.QPushButton(Form)
        self.pushButton_import.setObjectName("pushButton_import")
        self.gridLayout.addWidget(self.pushButton_import, 7, 0, 1, 3)
        self.label_current_position = QtWidgets.QLabel(Form)
        self.label_current_position.setText("")
        self.label_current_position.setObjectName("label_current_position")
        self.gridLayout.addWidget(self.label_current_position, 3, 1, 1, 2)
        self.pushButton_go_to = QtWidgets.QPushButton(Form)
        self.pushButton_go_to.setObjectName("pushButton_go_to")
        self.gridLayout.addWidget(self.pushButton_go_to, 3, 0, 1, 1)
        self.label_minimap = QtWidgets.QLabel(Form)
        self.label_minimap.setText("")
        self.label_minimap.setObjectName("label_minimap")
        self.gridLayout.addWidget(self.label_minimap, 8, 0, 1, 3)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label.setText(_translate("Form", "Positions "))
        self.pushButton_remove_position.setText(_translate("Form", "Remove Position"))
        self.pushButton_save_position.setText(_translate("Form", "Save position"))
        self.label_2.setText(_translate("Form", "Saved positions"))
        self.label_3.setText(_translate("Form", "All positions in mm and degrees"))
        self.pushButton_export.setText(_translate("Form", "Export positions"))
        self.pushButton_import.setText(_translate("Form", "Import Positions"))
        self.pushButton_go_to.setText(_translate("Form", "Go To"))
