# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ImageSettingsWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(535, 878)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.pushButton_take_image = QtWidgets.QPushButton(Form)
        self.pushButton_take_image.setObjectName("pushButton_take_image")
        self.gridLayout.addWidget(self.pushButton_take_image, 29, 0, 1, 2)
        self.pushButton_take_all_images = QtWidgets.QPushButton(Form)
        self.pushButton_take_all_images.setObjectName("pushButton_take_all_images")
        self.gridLayout.addWidget(self.pushButton_take_all_images, 30, 0, 1, 2)
        self.scrollArea = QtWidgets.QScrollArea(Form)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, -46, 501, 811))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_detector_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_detector_header.setFont(font)
        self.label_detector_header.setObjectName("label_detector_header")
        self.gridLayout_2.addWidget(self.label_detector_header, 26, 0, 1, 3)
        self.label_detector_contrast = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_contrast.setObjectName("label_detector_contrast")
        self.gridLayout_2.addWidget(self.label_detector_contrast, 30, 0, 1, 1)
        self.label_shift = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_shift.setObjectName("label_shift")
        self.gridLayout_2.addWidget(self.label_shift, 23, 0, 1, 1)
        self.doubleSpinBox_image_hfw = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.doubleSpinBox_image_hfw.setMaximum(100000000.0)
        self.doubleSpinBox_image_hfw.setSingleStep(0.1)
        self.doubleSpinBox_image_hfw.setProperty("value", 150.0)
        self.doubleSpinBox_image_hfw.setObjectName("doubleSpinBox_image_hfw")
        self.gridLayout_2.addWidget(self.doubleSpinBox_image_hfw, 5, 1, 1, 2)
        self.label_title = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout_2.addWidget(self.label_title, 2, 0, 1, 3)
        self.label_utils_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_utils_header.setFont(font)
        self.label_utils_header.setObjectName("label_utils_header")
        self.gridLayout_2.addWidget(self.label_utils_header, 33, 0, 1, 1)
        self.crosshair_checkbox = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.crosshair_checkbox.setChecked(True)
        self.crosshair_checkbox.setObjectName("crosshair_checkbox")
        self.gridLayout_2.addWidget(self.crosshair_checkbox, 34, 1, 1, 1)
        self.checkBox_image_save_image = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBox_image_save_image.setObjectName("checkBox_image_save_image")
        self.gridLayout_2.addWidget(self.checkBox_image_save_image, 8, 0, 1, 3)
        self.detector_brightness_slider = QtWidgets.QSlider(self.scrollAreaWidgetContents)
        self.detector_brightness_slider.setMaximum(100)
        self.detector_brightness_slider.setSliderPosition(50)
        self.detector_brightness_slider.setOrientation(QtCore.Qt.Horizontal)
        self.detector_brightness_slider.setInvertedAppearance(False)
        self.detector_brightness_slider.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.detector_brightness_slider.setTickInterval(10)
        self.detector_brightness_slider.setObjectName("detector_brightness_slider")
        self.gridLayout_2.addWidget(self.detector_brightness_slider, 29, 1, 1, 1)
        self.label_detector_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_type.setObjectName("label_detector_type")
        self.gridLayout_2.addWidget(self.label_detector_type, 27, 0, 1, 1)
        self.set_detector_button = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        self.set_detector_button.setObjectName("set_detector_button")
        self.gridLayout_2.addWidget(self.set_detector_button, 31, 0, 1, 3)
        self.detector_contrast_slider = QtWidgets.QSlider(self.scrollAreaWidgetContents)
        self.detector_contrast_slider.setMaximum(100)
        self.detector_contrast_slider.setSliderPosition(50)
        self.detector_contrast_slider.setOrientation(QtCore.Qt.Horizontal)
        self.detector_contrast_slider.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.detector_contrast_slider.setTickInterval(10)
        self.detector_contrast_slider.setObjectName("detector_contrast_slider")
        self.gridLayout_2.addWidget(self.detector_contrast_slider, 30, 1, 1, 1)
        self.label_image_hfw = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_image_hfw.setObjectName("label_image_hfw")
        self.gridLayout_2.addWidget(self.label_image_hfw, 5, 0, 1, 1)
        self.label_detector_mode = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_mode.setObjectName("label_detector_mode")
        self.gridLayout_2.addWidget(self.label_detector_mode, 28, 0, 1, 1)
        self.label_image_label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_image_label.setObjectName("label_image_label")
        self.gridLayout_2.addWidget(self.label_image_label, 11, 0, 1, 1)
        self.checkBox_advanced_settings = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBox_advanced_settings.setObjectName("checkBox_advanced_settings")
        self.gridLayout_2.addWidget(self.checkBox_advanced_settings, 0, 0, 1, 2)
        self.label_beam_header = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_beam_header.setFont(font)
        self.label_beam_header.setObjectName("label_beam_header")
        self.gridLayout_2.addWidget(self.label_beam_header, 18, 0, 1, 3)
        self.stigmation_y = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.stigmation_y.setMaximum(1000.0)
        self.stigmation_y.setObjectName("stigmation_y")
        self.gridLayout_2.addWidget(self.stigmation_y, 22, 2, 1, 1)
        self.spinBox_resolution_x = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        self.spinBox_resolution_x.setMinimum(1)
        self.spinBox_resolution_x.setMaximum(999999999)
        self.spinBox_resolution_x.setProperty("value", 1536)
        self.spinBox_resolution_x.setObjectName("spinBox_resolution_x")
        self.gridLayout_2.addWidget(self.spinBox_resolution_x, 3, 1, 1, 1)
        self.label_beam_working_distance = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_beam_working_distance.setObjectName("label_beam_working_distance")
        self.gridLayout_2.addWidget(self.label_beam_working_distance, 19, 0, 1, 1)
        self.scalebar_checkbox = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.scalebar_checkbox.setObjectName("scalebar_checkbox")
        self.gridLayout_2.addWidget(self.scalebar_checkbox, 34, 0, 1, 1)
        self.checkBox_image_use_autocontrast = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBox_image_use_autocontrast.setChecked(True)
        self.checkBox_image_use_autocontrast.setObjectName("checkBox_image_use_autocontrast")
        self.gridLayout_2.addWidget(self.checkBox_image_use_autocontrast, 7, 0, 1, 1)
        self.label_beam_current = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_beam_current.setObjectName("label_beam_current")
        self.gridLayout_2.addWidget(self.label_beam_current, 20, 0, 1, 1)
        self.ion_ruler_checkBox = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.ion_ruler_checkBox.setObjectName("ion_ruler_checkBox")
        self.gridLayout_2.addWidget(self.ion_ruler_checkBox, 34, 2, 1, 1)
        self.spinBox_beam_scan_rotation = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        self.spinBox_beam_scan_rotation.setMaximum(360)
        self.spinBox_beam_scan_rotation.setObjectName("spinBox_beam_scan_rotation")
        self.gridLayout_2.addWidget(self.spinBox_beam_scan_rotation, 24, 1, 1, 2)
        self.doubleSpinBox_image_dwell_time = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.doubleSpinBox_image_dwell_time.setMaximum(10000000000000.0)
        self.doubleSpinBox_image_dwell_time.setSingleStep(0.01)
        self.doubleSpinBox_image_dwell_time.setProperty("value", 1.0)
        self.doubleSpinBox_image_dwell_time.setObjectName("doubleSpinBox_image_dwell_time")
        self.gridLayout_2.addWidget(self.doubleSpinBox_image_dwell_time, 4, 1, 1, 2)
        self.button_set_beam_settings = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        self.button_set_beam_settings.setObjectName("button_set_beam_settings")
        self.gridLayout_2.addWidget(self.button_set_beam_settings, 25, 0, 1, 3)
        self.selected_beam = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        self.selected_beam.setObjectName("selected_beam")
        self.gridLayout_2.addWidget(self.selected_beam, 1, 1, 1, 2)
        self.shift_y = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.shift_y.setMinimum(-1000.0)
        self.shift_y.setMaximum(1000.0)
        self.shift_y.setObjectName("shift_y")
        self.gridLayout_2.addWidget(self.shift_y, 23, 2, 1, 1)
        self.comboBox_presets = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        self.comboBox_presets.setObjectName("comboBox_presets")
        self.gridLayout_2.addWidget(self.comboBox_presets, 6, 1, 1, 2)
        self.label_stigmation = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_stigmation.setObjectName("label_stigmation")
        self.gridLayout_2.addWidget(self.label_stigmation, 22, 0, 1, 1)
        self.detector_type_combobox = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        self.detector_type_combobox.setObjectName("detector_type_combobox")
        self.gridLayout_2.addWidget(self.detector_type_combobox, 27, 1, 1, 2)
        self.label_image_dwell_time = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_image_dwell_time.setObjectName("label_image_dwell_time")
        self.gridLayout_2.addWidget(self.label_image_dwell_time, 4, 0, 1, 1)
        self.label_presets = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_presets.setObjectName("label_presets")
        self.gridLayout_2.addWidget(self.label_presets, 6, 0, 1, 1)
        self.label_beam_scan_rotation = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_beam_scan_rotation.setObjectName("label_beam_scan_rotation")
        self.gridLayout_2.addWidget(self.label_beam_scan_rotation, 24, 0, 1, 1)
        self.label_detector_brightness = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_detector_brightness.setObjectName("label_detector_brightness")
        self.gridLayout_2.addWidget(self.label_detector_brightness, 29, 0, 1, 1)
        self.checkBox_image_use_autogamma = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBox_image_use_autogamma.setObjectName("checkBox_image_use_autogamma")
        self.gridLayout_2.addWidget(self.checkBox_image_use_autogamma, 7, 1, 1, 1)
        self.lineEdit_image_path = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_image_path.setObjectName("lineEdit_image_path")
        self.gridLayout_2.addWidget(self.lineEdit_image_path, 10, 1, 1, 2)
        self.detector_brightness_label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.detector_brightness_label.setObjectName("detector_brightness_label")
        self.gridLayout_2.addWidget(self.detector_brightness_label, 29, 2, 1, 1)
        self.spinBox_resolution_y = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        self.spinBox_resolution_y.setMinimum(1)
        self.spinBox_resolution_y.setMaximum(999999999)
        self.spinBox_resolution_y.setProperty("value", 1024)
        self.spinBox_resolution_y.setObjectName("spinBox_resolution_y")
        self.gridLayout_2.addWidget(self.spinBox_resolution_y, 3, 2, 1, 1)
        self.label_image_save_path = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_image_save_path.setObjectName("label_image_save_path")
        self.gridLayout_2.addWidget(self.label_image_save_path, 10, 0, 1, 1)
        self.lineEdit_image_label = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_image_label.setObjectName("lineEdit_image_label")
        self.gridLayout_2.addWidget(self.lineEdit_image_label, 11, 1, 1, 2)
        self.detector_mode_combobox = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        self.detector_mode_combobox.setObjectName("detector_mode_combobox")
        self.gridLayout_2.addWidget(self.detector_mode_combobox, 28, 1, 1, 2)
        self.label_image_resolution = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_image_resolution.setObjectName("label_image_resolution")
        self.gridLayout_2.addWidget(self.label_image_resolution, 3, 0, 1, 1)
        self.stigmation_x = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.stigmation_x.setMinimum(-1000.0)
        self.stigmation_x.setMaximum(1000.0)
        self.stigmation_x.setObjectName("stigmation_x")
        self.gridLayout_2.addWidget(self.stigmation_x, 22, 1, 1, 1)
        self.detector_contrast_label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.detector_contrast_label.setObjectName("detector_contrast_label")
        self.gridLayout_2.addWidget(self.detector_contrast_label, 30, 2, 1, 1)
        self.label_beam_voltage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_beam_voltage.setObjectName("label_beam_voltage")
        self.gridLayout_2.addWidget(self.label_beam_voltage, 21, 0, 1, 1)
        self.shift_x = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.shift_x.setMinimum(-1000.0)
        self.shift_x.setMaximum(1000.0)
        self.shift_x.setObjectName("shift_x")
        self.gridLayout_2.addWidget(self.shift_x, 23, 1, 1, 1)
        self.label_beam_type = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_beam_type.setObjectName("label_beam_type")
        self.gridLayout_2.addWidget(self.label_beam_type, 1, 0, 1, 1)
        self.ion_ruler_label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.ion_ruler_label.setText("")
        self.ion_ruler_label.setObjectName("ion_ruler_label")
        self.gridLayout_2.addWidget(self.ion_ruler_label, 33, 1, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 35, 0, 1, 2)
        self.working_distance = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.working_distance.setMaximum(1000.0)
        self.working_distance.setObjectName("working_distance")
        self.gridLayout_2.addWidget(self.working_distance, 19, 1, 1, 2)
        self.beam_current = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.beam_current.setMaximum(1000.0)
        self.beam_current.setObjectName("beam_current")
        self.gridLayout_2.addWidget(self.beam_current, 20, 1, 1, 2)
        self.beam_voltage = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.beam_voltage.setMaximum(100000.0)
        self.beam_voltage.setObjectName("beam_voltage")
        self.gridLayout_2.addWidget(self.beam_voltage, 21, 1, 1, 2)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 1, 0, 1, 2)
        self.pushButton_live_imaging = QtWidgets.QPushButton(Form)
        self.pushButton_live_imaging.setObjectName("pushButton_live_imaging")
        self.gridLayout.addWidget(self.pushButton_live_imaging, 31, 0, 1, 2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.pushButton_take_image, self.pushButton_take_all_images)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.pushButton_take_image.setText(_translate("Form", "Acquire Image"))
        self.pushButton_take_all_images.setText(_translate("Form", "Acquire All Images"))
        self.label_detector_header.setText(_translate("Form", "Detector Settings"))
        self.label_detector_contrast.setText(_translate("Form", "Contrast (%)"))
        self.label_shift.setText(_translate("Form", "Shift (x,y) (um)"))
        self.label_title.setText(_translate("Form", "Image Settings"))
        self.label_utils_header.setText(_translate("Form", "Utilities"))
        self.crosshair_checkbox.setText(_translate("Form", "Cross Hair"))
        self.checkBox_image_save_image.setText(_translate("Form", "Save Image"))
        self.label_detector_type.setText(_translate("Form", "Type"))
        self.set_detector_button.setText(_translate("Form", "Set Detector Settings"))
        self.label_image_hfw.setText(_translate("Form", "HFW (um)"))
        self.label_detector_mode.setText(_translate("Form", "Mode"))
        self.label_image_label.setText(_translate("Form", "Filename"))
        self.checkBox_advanced_settings.setText(_translate("Form", "Advanced Settings"))
        self.label_beam_header.setText(_translate("Form", "Beam Settings"))
        self.label_beam_working_distance.setText(_translate("Form", "Working Distance (mm)"))
        self.scalebar_checkbox.setText(_translate("Form", "Scale Bar"))
        self.checkBox_image_use_autocontrast.setText(_translate("Form", "AutoContrast"))
        self.label_beam_current.setText(_translate("Form", "Beam Current (pA)"))
        self.ion_ruler_checkBox.setText(_translate("Form", "Ruler "))
        self.button_set_beam_settings.setText(_translate("Form", "Set Beam Settings"))
        self.label_stigmation.setText(_translate("Form", "Stigmation (x,y)"))
        self.label_image_dwell_time.setText(_translate("Form", "Dwell Time (us)"))
        self.label_presets.setText(_translate("Form", "Preset"))
        self.label_beam_scan_rotation.setText(_translate("Form", "Scan Rotation (deg)"))
        self.label_detector_brightness.setText(_translate("Form", "Brightness (%)"))
        self.checkBox_image_use_autogamma.setText(_translate("Form", "AutoGamma"))
        self.detector_brightness_label.setText(_translate("Form", "50%"))
        self.label_image_save_path.setText(_translate("Form", "Path"))
        self.label_image_resolution.setText(_translate("Form", "Resolution (px)"))
        self.detector_contrast_label.setText(_translate("Form", "50%"))
        self.label_beam_voltage.setText(_translate("Form", "Voltage (kV)"))
        self.label_beam_type.setText(_translate("Form", "Beam"))
        self.pushButton_live_imaging.setText(_translate("Form", "Live Imaging"))
