# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FibsemModelTrainingWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(872, 888)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(Form)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 838, 478))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.spinBox_batch_size = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        self.spinBox_batch_size.setMaximum(256)
        self.spinBox_batch_size.setProperty("value", 4)
        self.spinBox_batch_size.setObjectName("spinBox_batch_size")
        self.gridLayout_6.addWidget(self.spinBox_batch_size, 10, 1, 1, 2)
        self.label_checkpoint = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_checkpoint.setObjectName("label_checkpoint")
        self.gridLayout_6.addWidget(self.label_checkpoint, 7, 0, 1, 1)
        self.lineEdit_save_path = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_save_path.setObjectName("lineEdit_save_path")
        self.gridLayout_6.addWidget(self.lineEdit_save_path, 4, 1, 1, 1)
        self.label_epochs = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_epochs.setObjectName("label_epochs")
        self.gridLayout_6.addWidget(self.label_epochs, 11, 0, 1, 1)
        self.doubleSpinBox_lr = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        self.doubleSpinBox_lr.setDecimals(4)
        self.doubleSpinBox_lr.setSingleStep(0.0001)
        self.doubleSpinBox_lr.setProperty("value", 0.0001)
        self.doubleSpinBox_lr.setObjectName("doubleSpinBox_lr")
        self.gridLayout_6.addWidget(self.doubleSpinBox_lr, 12, 1, 1, 2)
        self.lineEdit_data_path = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_data_path.setObjectName("lineEdit_data_path")
        self.gridLayout_6.addWidget(self.lineEdit_data_path, 2, 1, 1, 1)
        self.labelPath_button = QtWidgets.QToolButton(self.scrollAreaWidgetContents)
        self.labelPath_button.setObjectName("labelPath_button")
        self.gridLayout_6.addWidget(self.labelPath_button, 3, 2, 1, 1)
        self.label_num_classes = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_num_classes.setObjectName("label_num_classes")
        self.gridLayout_6.addWidget(self.label_num_classes, 8, 0, 1, 1)
        self.comboBox_encoder = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        self.comboBox_encoder.setObjectName("comboBox_encoder")
        self.gridLayout_6.addWidget(self.comboBox_encoder, 6, 1, 1, 2)
        self.lineEdit_checkpoint = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_checkpoint.setObjectName("lineEdit_checkpoint")
        self.gridLayout_6.addWidget(self.lineEdit_checkpoint, 7, 1, 1, 2)
        self.label_wandb_entity = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_wandb_entity.setObjectName("label_wandb_entity")
        self.gridLayout_6.addWidget(self.label_wandb_entity, 15, 0, 1, 1)
        self.spinBox_num_classes = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        self.spinBox_num_classes.setObjectName("spinBox_num_classes")
        self.gridLayout_6.addWidget(self.spinBox_num_classes, 8, 1, 1, 2)
        self.label_encoder = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_encoder.setObjectName("label_encoder")
        self.gridLayout_6.addWidget(self.label_encoder, 6, 0, 1, 1)
        self.checkBox_use_wandb = QtWidgets.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBox_use_wandb.setObjectName("checkBox_use_wandb")
        self.gridLayout_6.addWidget(self.checkBox_use_wandb, 14, 0, 1, 1)
        self.label_label_path = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_label_path.setObjectName("label_label_path")
        self.gridLayout_6.addWidget(self.label_label_path, 3, 0, 1, 1)
        self.label_data_path = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_data_path.setObjectName("label_data_path")
        self.gridLayout_6.addWidget(self.label_data_path, 2, 0, 1, 1)
        self.outputPath_button = QtWidgets.QToolButton(self.scrollAreaWidgetContents)
        self.outputPath_button.setObjectName("outputPath_button")
        self.gridLayout_6.addWidget(self.outputPath_button, 4, 2, 1, 1)
        self.label_learning_rate = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_learning_rate.setObjectName("label_learning_rate")
        self.gridLayout_6.addWidget(self.label_learning_rate, 12, 0, 1, 1)
        self.lineEdit_label_path = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_label_path.setObjectName("lineEdit_label_path")
        self.gridLayout_6.addWidget(self.lineEdit_label_path, 3, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label.setObjectName("label")
        self.gridLayout_6.addWidget(self.label, 4, 0, 1, 1)
        self.label_batch_size = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_batch_size.setObjectName("label_batch_size")
        self.gridLayout_6.addWidget(self.label_batch_size, 10, 0, 1, 1)
        self.spinBox_epochs = QtWidgets.QSpinBox(self.scrollAreaWidgetContents)
        self.spinBox_epochs.setMaximum(9999)
        self.spinBox_epochs.setProperty("value", 20)
        self.spinBox_epochs.setObjectName("spinBox_epochs")
        self.gridLayout_6.addWidget(self.spinBox_epochs, 11, 1, 1, 2)
        self.dataPath_button = QtWidgets.QToolButton(self.scrollAreaWidgetContents)
        self.dataPath_button.setObjectName("dataPath_button")
        self.gridLayout_6.addWidget(self.dataPath_button, 2, 2, 1, 1)
        self.label_wandb_project = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_wandb_project.setObjectName("label_wandb_project")
        self.gridLayout_6.addWidget(self.label_wandb_project, 16, 0, 1, 1)
        self.lineEdit_wandb_entity = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_wandb_entity.setObjectName("lineEdit_wandb_entity")
        self.gridLayout_6.addWidget(self.lineEdit_wandb_entity, 15, 1, 1, 1)
        self.lineEdit_wandb_project = QtWidgets.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEdit_wandb_project.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.lineEdit_wandb_project.setObjectName("lineEdit_wandb_project")
        self.gridLayout_6.addWidget(self.lineEdit_wandb_project, 16, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout_6.addWidget(self.label_2, 0, 0, 1, 3)
        self.label_3 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.gridLayout_6.addWidget(self.label_3, 5, 0, 1, 3)
        self.label_4 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.gridLayout_6.addWidget(self.label_4, 9, 0, 1, 3)
        self.label_5 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.gridLayout_6.addWidget(self.label_5, 13, 0, 1, 3)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 1, 0, 1, 3)
        self.label_info = QtWidgets.QLabel(Form)
        self.label_info.setObjectName("label_info")
        self.gridLayout.addWidget(self.label_info, 26, 0, 1, 2)
        self.pushButton_train_model = QtWidgets.QPushButton(Form)
        self.pushButton_train_model.setObjectName("pushButton_train_model")
        self.gridLayout.addWidget(self.pushButton_train_model, 25, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 0, 1, 3)
        self.label_title = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_title.setFont(font)
        self.label_title.setObjectName("label_title")
        self.gridLayout.addWidget(self.label_title, 0, 0, 1, 3)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_checkpoint.setText(_translate("Form", "Checkpoint"))
        self.label_epochs.setText(_translate("Form", "Epochs"))
        self.labelPath_button.setText(_translate("Form", "..."))
        self.label_num_classes.setText(_translate("Form", "Num Classes"))
        self.label_wandb_entity.setText(_translate("Form", "Entity"))
        self.label_encoder.setText(_translate("Form", "Encoder"))
        self.checkBox_use_wandb.setText(_translate("Form", "Use Weights and Biases"))
        self.label_label_path.setText(_translate("Form", "Label Path"))
        self.label_data_path.setText(_translate("Form", "Data Path"))
        self.outputPath_button.setText(_translate("Form", "..."))
        self.label_learning_rate.setText(_translate("Form", "Learning Rate"))
        self.label.setText(_translate("Form", "Save Path"))
        self.label_batch_size.setText(_translate("Form", "Batch Size"))
        self.dataPath_button.setText(_translate("Form", "..."))
        self.label_wandb_project.setText(_translate("Form", "Project"))
        self.label_2.setText(_translate("Form", "Data"))
        self.label_3.setText(_translate("Form", "Model"))
        self.label_4.setText(_translate("Form", "Hyper Parameters"))
        self.label_5.setText(_translate("Form", "Logging"))
        self.label_info.setText(_translate("Form", "Info"))
        self.pushButton_train_model.setText(_translate("Form", "Train Model"))
        self.label_title.setText(_translate("Form", "Model Training"))
