# style sheets
# TODO: TEMPLATE THIS CSS   
_GREEN_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: green;
    }
QPushButton:hover {
    background-color: rgba(0, 255, 0, 125);
    }"""

_RED_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: red;
    }
QPushButton:hover {
    background-color: rgba(255, 0, 0, 125);
    }"""

_BLUE_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: blue;
    }
QPushButton:hover {
    background-color: rgba(0, 0, 255, 125);
    }"""

_YELLOW_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: yellow;
    }
QPushButton:hover {
    background-color: rgba(255, 255, 0, 125);
    }"""

_WHITE_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: white;
    }
QPushButton:hover {
    background-color: rgba(255, 255, 255, 125);
    }"""

_GRAY_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: gray;
    }
QPushButton:hover {
    background-color: rgba(125, 125, 125, 125);
    }"""

_ORANGE_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: orange;
    }
QPushButton:hover {
    background-color: rgba(255, 125, 0, 125);
}"""

_DISABLED_PUSHBUTTON_STYLE = """
QPushButton {
    background-color: none;
    }
"""    